package com.bocloud.cms.booter.config;

import com.bocloud.cms.booter.listener.AlarmListener;
import com.bocloud.cms.booter.listener.ResourceDeletedListener;
import com.bocloud.cms.booter.listener.SoftwareEventListenner;
import com.bocloud.cms.booter.listener.VmReclaimListener;
import com.megatron.common.utils.MapTools;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import java.util.Map;

/**
 * rabbitmq相关配置
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年4月16日
 */
@Configuration
@ConfigurationProperties(prefix = "spring.rabbitmq")
public class RabbitMqConfiguration {

    private final Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");

    @Autowired
    private ConnectionFactory connectionFactory;

    @Bean
    public Queue declareAlarmQueue() {
        return new Queue("resource.alarm.queue", true, false, false, arguments);
    }

    @Bean
    public Queue declareCmpResourceDeletedQueue() {
        return new Queue("cmp.resource.deleted.queue", true, false, false, arguments);
    }

    @Bean
    public Queue declareCmpVmReclaimQueue() {
        return new Queue("cmp.vm.reclaim.queue", true, false, false, arguments);
    }

    @Bean
    public Queue softwareEventQueue() {
        return new Queue("software.event.queue", true, false, false, arguments);
    }

    @Bean
    public DirectExchange declareDirectExchange() {
        return new DirectExchange("bocloud.direct.exchange", true, true, arguments);
    }

    @Bean
    public Binding bindingAlarmQueue() {
        return BindingBuilder.bind(declareAlarmQueue()).to(declareDirectExchange()).withQueueName();
    }

    @Bean
    public Binding bindingCmpResourceDeletedQueue() {
        return BindingBuilder.bind(declareCmpResourceDeletedQueue()).to(declareDirectExchange()).withQueueName();
    }

    @Bean
    public Binding bindingCmpVmReclaimQueue() {
        return BindingBuilder.bind(declareCmpVmReclaimQueue()).to(declareDirectExchange()).withQueueName();
    }

    @Bean
    public Binding bindingSoftwareEventQueue() {
        return BindingBuilder.bind(softwareEventQueue()).to(declareDirectExchange()).withQueueName();
    }

    @Bean
    public SimpleMessageListenerContainer autoAlarmListener(AlarmListener alarmListener) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(declareAlarmQueue());
        // 设置确认模式自动确认
        container.setAcknowledgeMode(AcknowledgeMode.AUTO);
        container.setMessageListener(alarmListener);
        return container;
    }

    @Bean
    public SimpleMessageListenerContainer autoCmpResourceDeletedListener(ResourceDeletedListener resourceDeletedListener) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(declareCmpResourceDeletedQueue());
        // 设置确认模式自动确认
        container.setAcknowledgeMode(AcknowledgeMode.AUTO);
        container.setMessageListener(resourceDeletedListener);
        return container;
    }

    @Bean
    public SimpleMessageListenerContainer autoCmpVmReclaimListener(VmReclaimListener vmReclaimListener) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(declareCmpVmReclaimQueue());
        // 设置确认模式自动确认
        container.setAcknowledgeMode(AcknowledgeMode.AUTO);
        container.setMessageListener(vmReclaimListener);
        return container;
    }

    @Bean
    public SimpleMessageListenerContainer autoSoftwareEventQueueListener(SoftwareEventListenner softwareEventListenner) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(softwareEventQueue());
        // 设置确认模式自动确认
        container.setAcknowledgeMode(AcknowledgeMode.AUTO);
        container.setMessageListener(softwareEventListenner);
        return container;
    }


    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitTemplate amqpTemplate() {
        RabbitTemplate amqpTemplate = new RabbitTemplate(connectionFactory);
        amqpTemplate.setMessageConverter(messageConverter());
        return amqpTemplate;
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitAdmin rabbitAdmin() {
        return new RabbitAdmin(connectionFactory);
    }
}
