package com.bocloud.cms.booter.controller;

import com.bocloud.cms.model.PrometheusAlarm;
import com.megatron.common.model.GeneralResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 告警
 *
 * @author zhangyf
 * @version 5.5.0
 * @date 2022/2/21 18:01
 */
@Tag(name = "告警")
@RestController
@RequestMapping
public class AlarmReceiveController {

    private final ThreadPoolTaskExecutor taskExecutor;
    @Autowired
    private AmqpTemplate amqpTemplate;

    public AlarmReceiveController(ThreadPoolTaskExecutor taskExecutor) {
        this.taskExecutor = taskExecutor;
    }

    /**
     * mq接受告警信息
     *
     * @param prometheusAlarm prometheus获取的参数
     * @return 是否成功接受
     */
    @PostMapping("/v1/monitor/alarms")
    @Operation(summary = "告警接收")
    public GeneralResult receive(@RequestBody PrometheusAlarm prometheusAlarm) {
        taskExecutor.submit(() -> amqpTemplate.convertAndSend("resource.alarm.queue", prometheusAlarm));
        return new GeneralResult<>(true, "has been received");
    }

}
