package com.bocloud.cms.booter.controller;

import com.bocloud.cms.common.utils.ExcelChartUtil;
import com.bocloud.cms.intf.AnalysisService;
import com.bocloud.cms.intf.AnalysisSnapshotService;
import com.bocloud.cms.model.SnapshotParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

/**
 * 监控分析
 *
 * @author zhangyf
 * @since 2019-08-16
 */
@RestController
@Tag(name = "监控分析快照")
@RequestMapping("/v1/analysis/snapshots")
public class AnalysisSnapshotController {

    @Autowired
    private AnalysisSnapshotService analysisSnapshotService;
    @Autowired
    private AnalysisService analysisService;

    /**
     * 监控分析快照列表
     *
     * @param pager   分页
     * @param simple  默认
     * @param context 用户上下文
     * @return 数据列表
     */
    @GetMapping
    @Operation(summary = "监控分析快照列表")
    public GeneralResult list(Pager pager,
                              @RequestParam(value = Common.SIMPLE, required = false) Boolean simple,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisSnapshotService.list(pager, context);
    }

    /**
     * 删除分析快照
     *
     * @param id      快照id
     * @param context 用户上下文
     * @return 是否删除成功
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除监控分析快照")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisSnapshotService.remove(id, context.getTarget());
    }

    /**
     * 监控分析快照详情
     *
     * @param id 快照id
     * @return 快照实体类详情
     */
    @GetMapping("/{id}")
    @Operation(summary = "监控分析快照详情")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id) {
        return analysisSnapshotService.detail(id);
    }

    /**
     * 监控分析
     *
     * @param snapshotParam 请求具体参数
     * @param context       当前用户上下文
     * @return 执行任务是否成功
     */
    @PostMapping
    @Operation(summary = "执行监控分析任务")
    public GeneralResult handle(@RequestBody SnapshotParam snapshotParam,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return analysisService.handle(snapshotParam.getId(), snapshotParam.getVendorId(), snapshotParam.getStartTime(),
                snapshotParam.getEndTime(), context, request.getHeader(Common.TOKEN));
    }

    /**
     * 监控报表下载
     *
     * @param id       具体监控id
     * @param request  请求request
     * @param response 请求response
     * @return 下载文件
     */
    @GetMapping("/{id}/download")
    @Operation(summary = "监控报表下载")
    public GeneralResult download(@PathVariable(value = Common.ID) Long id, HttpServletRequest request, HttpServletResponse response) {
        GeneralResult result = analysisSnapshotService.detail(id);
        Assert.isTrue(result.isSuccess(), "无法创建报表:" + result.getMessage());
        return ExcelChartUtil.createMonitorCharExcel(result, request, response);
    }

}
