package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.CmsService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 消息队列监控
 *
 * @author zhangdi
 * @since Apr 7th,2020
 */

@RestController
@Tag(name = "消息队列监控")
@RequestMapping("/v1/cms")
public class CmsController {

    private final CmsService cmsService;

    @Autowired
    public CmsController(CmsService cmsService) {
        this.cmsService = cmsService;
    }

    /**
     * 消息队列列表
     *
     * @param pager   分页
     * @param catalog 标签
     * @param context 用户上下文
     * @param request 请求request
     * @return 消息队列列表
     */
    @GetMapping
    @Operation(summary = "消队列列表")
    public GeneralResult list(Pager pager, String catalog, @Value(Common.REQ_CONTEXT) RequestContext context,
                              HttpServletRequest request) {
        return cmsService.list(pager, catalog, context, request);
    }

    /**
     * Kafka Topic列表
     *
     * @param id 主题id
     * @return 是否接受到消息
     */
    @GetMapping("/kafka/{id}/topics")
    @Operation(summary = "Kafka Topic列表")
    public GeneralResult topicsOfKafka(@PathVariable(value = Common.ID) Long id) {
        return new GeneralResult<>(true, cmsService.topicsOfKafka(id), "SUCCESS");
    }

    /**
     * 获取对接消费消息
     *
     * @param id 队列id
     * @return 获取消息是否成功
     */
    @GetMapping("/kafka/{id}/consumergroups")
    @Operation(summary = "Kafka消费组列表")
    public GeneralResult consumerGroupsOfKafka(@PathVariable(value = Common.ID) Long id) {
        return new GeneralResult<>(true, cmsService.consumerGroupsOfKafka(id), "SUCCESS");
    }
}
