package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.MonitorHostIpmi;
import com.bocloud.cms.intf.HostService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 宿主机监控
 *
 * @author zhangdi
 * @since June 12,2019
 */

@RestController
@Tag(name = "宿主机监控")
@RequestMapping("/v1/hosts")
public class HostController {

    @Autowired
    private HostService hostService;

    /**
     * 宿主机列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 宿主机列表
     */
    @GetMapping
    @Operation(summary = "宿主机列表")
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return hostService.list(pager, context);
    }

    /**
     * 宿主机基本信息详情
     *
     * @param id   宿主机id
     * @param type 类型
     * @return 宿主机详情
     */
    @GetMapping("/{id}")
    @Operation(summary = "宿主机基本信息详情")
    public GeneralResult hostOverview(@PathVariable(value = Common.ID) Long id,
                                      @RequestParam(value = "type") String type) {
        return hostService.hostOverview(id, type);
    }

    /**
     * 宿主机Cpu或内存或磁盘信息
     *
     * @param id   宿主机id
     * @param type 类型
     * @return 主机硬件信息
     */
    @GetMapping("/{id}/metrics")
    @Operation(summary = "主机Cpu/内存/磁盘基本信息")
    public GeneralResult hostInfo(@PathVariable(value = Common.ID) Long id,
                                  @RequestParam(value = "type") String type) {
        return hostService.hostInfo(id, type);
    }

    /**
     * 配置宿主机ipmi信息
     *
     * @param hostIpmi 配置参数
     * @return 是否配置成功
     */
    @PatchMapping("/ipmi/binding")
    @Operation(summary = "宿主机绑定ipmi信息")
    public GeneralResult binding(@RequestBody MonitorHostIpmi hostIpmi,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return hostService.binding(hostIpmi, context);
    }

    /**
     * 删除宿主机ipmi信息
     *
     * @param id      宿主机id
     * @param context 当前用户上下文
     * @return 是否删除配置成功
     */
    @PatchMapping("/ipmi/unbind")
    @Operation(summary = "删除宿主机ipmi信息")
    public GeneralResult unbind(Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return hostService.unbind(id, context.getTarget());
    }

    /**
     * 获取宿主机ipmi信息
     *
     * @param id ID
     * @return 宿主机ipmi信息
     */
    @GetMapping("/ipmi/{id}")
    @Operation(summary = "宿主机ipmi信息详情")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id) {
        return hostService.detail(id);
    }


}
