package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.AlarmSendPolicy;
import com.bocloud.cms.intf.AlarmSendPolicyService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author lyy
 * @version 1.0
 * @since 2017年7月13日
 */
@RestController
@Tag(name = "分发策略")
@RequestMapping("/v1/distributions")
public class SendPolicyController {

    private final AlarmSendPolicyService alarmSendPolicyService;

    @Autowired
    public SendPolicyController(AlarmSendPolicyService alarmSendPolicyService) {
        this.alarmSendPolicyService = alarmSendPolicyService;
    }


    /**
     * 新增分发策略
     *
     * @param alarmSendPolicy 策略内容
     * @param context         用户上下文
     * @return 成功/失败
     */
    @PostMapping
    @Operation(summary = "新增分发策略")
    public GeneralResult create(@RequestBody AlarmSendPolicy alarmSendPolicy,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmSendPolicyService.create(alarmSendPolicy, context);
    }


    /**
     * 修改分发策略
     *
     * @param alarmSendPolicy 修改的策略内容
     * @param context         用户上下文
     * @return 成功/失败
     */
    @PutMapping("/{id}")
    @Operation(summary = "修改分发策略")
    public GeneralResult modify(@RequestBody AlarmSendPolicy alarmSendPolicy,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmSendPolicyService.modify(alarmSendPolicy, context);
    }

    /**
     * 删除分发策略
     *
     * @param id      策略id
     * @param context 用户上下文
     * @return 成功/失败
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除分发策略")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmSendPolicyService.remove(id, context);
    }

    /**
     * 获取应用列表
     *
     * @param pager   分页
     * @param context 当前用户上下文 操作者信息
     * @return 数据结果集
     */
    @GetMapping
    @Operation(summary = "分发策略列表")
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmSendPolicyService.list(pager, context);
    }


    /**
     * 分发策略详情
     *
     * @param id 策略id
     * @return 策略详情
     */
    @GetMapping("/{id}")
    @Operation(summary = "分发策略详情")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id) {
        return alarmSendPolicyService.detail(id);
    }


}
