package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.VmService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 云主机监控
 *
 * @author zhangdi
 * @since June 12,2019
 */
@Tag(name = "云主机监控")
@RestController
@RequestMapping("/v1/vms")
public class VmController {

    private final VmService vmService;

    public VmController(VmService vmService) {
        this.vmService = vmService;
    }

    /**
     * 云主机列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @param request HttpServletRequest
     * @return 云主机列表
     */
    @GetMapping
    @Operation(summary = "云主机列表")
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return vmService.list(pager, context, request);
    }

    /**
     * 云主机资源概览
     *
     * @param id 云主机ID
     * @return 云主机资源概览
     */
    @GetMapping("/{id}")
    @Operation(summary = "云主机资源概览")
    public GeneralResult vmOverview(@PathVariable(value = Common.ID) Long id) {
        return vmService.vmOverview(id);
    }


}
