package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.*;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * xsky监控
 *
 * @author zhangyf
 * @since 2020-07-08
 */
@Tag(name = "XSKY存储监控")
@RestController
@RequestMapping("/v1/xsky")
public class XskyController {

    private final XskyBucketService xskyBucketService;
    private final XskyObjectUserService xskyObjectUserService;
    private final XskyObjectRouterService xskyObjectRouterService;
    private final XskyDiskService xskyDiskService;
    private final XskyVolumeService xskyVolumeService;
    private final XskyVolumePoolService xskyVolumePoolService;
    private final XskyHostService xskyHostService;
    private final XskyCephOsdService xskyCephOsdService;
    private final XskyFileSystemService xskyFileSystemService;

    @Autowired
    public XskyController(XskyBucketService xskyBucketService, XskyObjectUserService xskyObjectUserService, XskyObjectRouterService xskyObjectRouterService, XskyDiskService xskyDiskService, XskyVolumeService xskyVolumeService, XskyVolumePoolService xskyVolumePoolService, XskyHostService xskyHostService, XskyCephOsdService xskyCephOsdService, XskyFileSystemService xskyFileSystemService) {
        this.xskyBucketService = xskyBucketService;
        this.xskyObjectUserService = xskyObjectUserService;
        this.xskyObjectRouterService = xskyObjectRouterService;
        this.xskyDiskService = xskyDiskService;
        this.xskyVolumeService = xskyVolumeService;
        this.xskyVolumePoolService = xskyVolumePoolService;
        this.xskyHostService = xskyHostService;
        this.xskyCephOsdService = xskyCephOsdService;
        this.xskyFileSystemService = xskyFileSystemService;
    }

    /**
     * 服务器列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 服务器列表
     */
    @GetMapping("/hosts")
    @Operation(summary = "服务器列表")
    public GeneralResult hosts(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyHostService.list(pager, context);
    }


    /**
     * 对象用户列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 对象用户列表
     */
    @GetMapping("/object/user")
    @Operation(summary = "对象用户列表")
    public GeneralResult objectUsers(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyObjectUserService.list(pager, context);
    }

    /**
     * 对象路由列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 对象路由列表
     */
    @GetMapping("/object/router")
    @Operation(summary = "对象路由列表")
    public GeneralResult objectRouters(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyObjectRouterService.list(pager, context);
    }

    /**
     * 存储桶列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 存储桶列表
     */
    @GetMapping("/buckets")
    @Operation(summary = "存储桶列表")
    public GeneralResult buckets(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyBucketService.list(pager, context);
    }

    /**
     * 存储池列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 存储池列表
     */
    @GetMapping("/storages/pools")
    @Operation(summary = "存储池列表")
    public GeneralResult volumePools(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyVolumePoolService.list(pager, context);
    }

    /**
     * 硬盘列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 硬盘列表
     */
    @GetMapping("/ceph/osds")
    @Operation(summary = "硬盘列表")
    public GeneralResult osds(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyCephOsdService.list(pager, context);
    }

    /**
     * 缓存磁盘列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 缓存磁盘列表
     */
    @GetMapping("/disks")
    @Operation(summary = "缓存磁盘列表")
    public GeneralResult disks(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyDiskService.list(pager, context);
    }

    /**
     * 存储卷列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 存储卷列表
     */
    @GetMapping("/volumes")
    @Operation(summary = "存储卷列表")
    public GeneralResult volumes(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyVolumeService.list(pager, context);
    }

    /**
     * 文件系统列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 文件系统列表
     */
    @GetMapping("/fileSystems")
    @Operation(summary = "文件系统列表")
    public GeneralResult fileSystems(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyFileSystemService.list(pager, context);
    }


}
