package com.bocloud.cms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

/**
 * 回收租户云主机
 *
 * @author zhangyf
 * @since 2019-12-05
 */
@Component
@Slf4j
public class VmReclaimListener implements MessageListener {


    /**
     * @param message {
     *                "data": 云主机entity对象
     *                }
     */
    @RabbitListener(queues = "cmp.vm.reclaim.queue", admin = "rabbitAdmin", autoStartup = "true")
    @Override
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            JSONObject jsonObject = JSONObject.parseObject(content);
            CloudServer cloudServer = jsonObject.getObject("data", CloudServer.class);
            MonitorResourceType resourceType;
            switch (CloudProvider.valueOf(cloudServer.getVendorType())) {
                case VMWARE:
                    resourceType = MonitorResourceType.MONITOR_VCENTER_VM;
                    break;
                case OPENSTACK:
                    resourceType = MonitorResourceType.MONITOR_OPENSTACK_VM;
                    break;
                case MANAGEONE:
                case FUSIONCLOUD:
                    resourceType = MonitorResourceType.MONITOR_FUSIONCLOUD_VM;
                    break;
                case CTSTACK:
                    resourceType = MonitorResourceType.MONITOR_CTSTACK_VM;
                    break;
                case HUAWEI:
                    resourceType = MonitorResourceType.MONITOR_HUAWEI_VM;
                    break;
                case CAS:
                    resourceType = MonitorResourceType.MONITOR_H3C_VM;
                    break;
                case ALIYUN:
                    resourceType = MonitorResourceType.MONITOR_ALIYUN_VM;
                    break;
                case KINGCLOUD:
                    resourceType = MonitorResourceType.MONITOR_KINGCLOUD_VM;
                    break;
                default:
                    return;
            }
            //TODO
        } catch (Exception e) {
            log.error("failed to delete rule relation data after reclaim tenant virtual machine：{}", e.getMessage(), e);
        }
    }
}
