package com.bocloud.cms.common.utils;

import java.io.Serializable;
import java.sql.Blob;
import java.sql.Clob;
import java.util.Collection;
import java.util.HashMap;
import java.util.Set;

/**
 * map工具类
 * 把请求的所有参数封装到Map中,提供最常用的方法
 * @author lyy
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.5.0
 */
public class BaseFormMap<K, V> extends HashMap<K, V> implements Serializable {

    private static final long serialVersionUID = 1L;

    @SuppressWarnings("unchecked")
    public void set(K key, V value) {
        if (key != null) {
            key = (K) key.toString().toUpperCase();
        }
        super.put(key, value);
    }

    @SuppressWarnings("unchecked")
    @Override
    public V put(K key, V value) {
        if (key != null) {
            key = (K) key.toString().toUpperCase();
        }
        return super.put(key, value);
    }

    /**
     * 取值一律转成大写
     *
     * @param key
     * @return
     */
    @Override
    public V get(Object key) {
        if (key != null) {
            key = key.toString().toUpperCase();
        }
        return super.get(key);
    }

    /**
     * Get attribute of mysql type: varchar, char, enum, set, text, tinytext,
     * mediumtext, longtext
     */
    public String getStr(String attr) {
        return this.get(attr) == null ? "" : (String) this.get(attr);
    }

    /**
     * Get attribute of mysql type: int, integer, tinyint(n) n > 1, smallint,
     * mediumint
     */
    public Integer getInt(String attr) {
        return (Integer) this.get(attr);
    }

    /**
     * Get attribute of mysql type: bigint, unsign int
     */
    public Long getLong(String attr) {
        return (Long) this.get(attr);
    }

    /**
     * Get attribute of mysql type: unsigned bigint
     */
    public java.math.BigInteger getBigInteger(String attr) {
        return (java.math.BigInteger) this.get(attr);
    }

    /**
     * Get attribute of mysql type: date, year
     */
    public java.util.Date getDate(String attr) {
        return (java.util.Date) this.get(attr);
    }

    /**
     * Get attribute of mysql type: time
     */
    public java.sql.Time getTime(String attr) {
        return (java.sql.Time) this.get(attr);
    }

    /**
     * Get attribute of mysql type: timestamp, datetime
     */
    public java.sql.Timestamp getTimestamp(String attr) {
        return (java.sql.Timestamp) this.get(attr);
    }

    /**
     * Get attribute of mysql type: real, double
     */
    public Double getDouble(String attr) {
        return (Double) this.get(attr);
    }

    /**
     * Get attribute of mysql type: float
     */
    public Float getFloat(String attr) {
        return (Float) this.get(attr);
    }

    /**
     * Get attribute of mysql type: bit, tinyint(1)
     */
    public Boolean getBoolean(String attr) {
        return (Boolean) this.get(attr);
    }

    /**
     * Get attribute of mysql type: decimal, numeric
     */
    public java.math.BigDecimal getBigDecimal(String attr) {
        return (java.math.BigDecimal) this.get(attr);
    }

    /**
     * Get attribute of mysql type: binary, varbinary, tinyblob, blob,
     * mediumblob, longblob
     */
    public byte[] getBytes(String attr) {
        return (byte[]) this.get(attr);
    }

    public Blob getBlob(String attr) {
        return (Blob) this.get(attr);
    }

    /**
     * Get attribute of any type that extends from Number
     */
    public Number getNumber(String attr) {
        return (Number) this.get(attr);
    }

    public Clob getClob(String attr) {
        return (Clob) this.get(attr);
    }


    /**
     * Return attribute names of this model.
     */
    public String[] getAttrNames() {
        Set<K> attrNameSet = this.keySet();
        return attrNameSet.toArray(new String[attrNameSet.size()]);
    }

    /**
     * Return attribute values of this model.
     */
    public Object[] getAttrValues() {
        Collection<V> attrValueCollection = values();
        return attrValueCollection.toArray(new Object[attrValueCollection
                .size()]);
    }

}
