package com.bocloud.cms.common.utils;

import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.sshd.client.SshClient;
import org.apache.sshd.client.channel.ClientChannel;
import org.apache.sshd.client.channel.ClientChannelEvent;
import org.apache.sshd.client.session.ClientSession;
import org.apache.sshd.common.SshException;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * sshd工具类
 *
 * @author zhangyf
 * @since 2022-11-17
 */
@Slf4j
public class SshdExecutor {

    private final SshClient client;
    private final ClientSession session;

    @Getter
    private String result;
    @Getter
    private String error;

    public static void main(String[] args) throws IOException {
        SshdExecutor executor = new SshdExecutor("10.20.12.64", 22, "root", "boyun@1231");
        executor.execute("ls");
        System.out.println("1");
    }

    public SshdExecutor(String ip, Integer port, String user) throws IOException {
        client = SshClient.setUpDefaultClient();
        client.start();
        session = client.connect(user, ip, port).verify(10 * 1000).getSession();
    }

    public SshdExecutor(String ip, Integer port, String user, String password) throws IOException {
        this(ip, port, user);
        session.addPasswordIdentity(password);
    }

    public void execute(String command) {
        try {
            try {
                if (!session.auth().verify(10 * 1000).isSuccess()) {
                    throw new RuntimeException("认证失败");
                }
            } catch (SshException e) {
                log.error(e.getMessage(), e);
                throw new RuntimeException("认证失败, 请检查用户名密码是否正确", e);
            }
            ClientChannel channel = session.createExecChannel(command);
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            ByteArrayOutputStream err = new ByteArrayOutputStream();
            channel.setOut(out);
            channel.setErr(err);

            if (!channel.open().verify(10 * 1000).isOpened()) {
                throw new RuntimeException("open failed");
            }
            List<ClientChannelEvent> list = new ArrayList<>();
            list.add(ClientChannelEvent.CLOSED);
            channel.waitFor(list, 10 * 1000);
            channel.close();
            result = out.toString();
            error = err.toString();
            log.debug("result :" + error);
            if (StringUtils.isNotBlank(error)) {
                log.debug("error :" + error);
                throw new RuntimeException("execute shell failed：" + error);
            }
        } catch (RuntimeException | IOException e) {
            throw new RuntimeException(e.getMessage(), e);
        } finally {
            if (client != null) {
                try {
                    client.stop();
                    client.close();
                } catch (IOException e) {
                    log.warn(e.getMessage(), e);
                }
            }
        }
    }
}