package com.bocloud.cms.entity;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 告警实体类
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年7月10日
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Table("monitor_alarm")
public class Alarm extends GenericEntity {

    /**
     * ID
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 警告等级
     */
    @Column("level")
    private String level;
    /**
     * 目标机器类型
     */
    @Column("type")
    private String type;
    /**
     * 目标机器类型
     */
    @Column("os_category")
    private String osCategory;
    /**
     * 目标机器
     */
    @Column("target")
    private String target;
    /**
     * 处理结果
     */
    @Column("deal_result")
    private String dealResult;
    /**
     * 告警的key值
     */
    @Column("alarm_val")
    private String alarmVal;
    /**
     * 告警恢复的值
     */
    @Column("recovery_val")
    private String recoveryVal;
    /**
     * 告警触发时间
     */
    @Column("gmt_trigger")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtTrigger;
    /**
     * 告警确认时间
     */
    @Column("gmt_confirm")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtConfirm;
    /**
     * 告警确认人
     */
    @Column("confirmer")
    private String confirmer;
    /**
     * 告警解决时间
     */
    @Column("gmt_solve")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtSolve;
    /**
     * 告警解决人
     */
    @Column("solver")
    private String solver;
    /**
     * 云供应商Id
     */
    @Column("vendor_id")
    private Long vendorId;
    /**
     * 告警规则ID
     */
    @Column("rule_id")
    private Long ruleId;
    /**
     * 是否是事件告警
     */
    @Column("event_alarm")
    private Boolean eventAlarm;
    /**
     * 是否已经发送通知
     */
    @Column("send_notice")
    private Boolean sendNotice;
    /**
     * 是否已经发送告警恢复通知
     */
    @Column("send_recovery")
    private Boolean sendRecovery;
    /**
     * 项目ID
     */
    @Column("tenant_id")
    private Long tenantId;
    /**
     * 告警目标名称
     */
    @Column("target_name")
    private String targetName;
    /**
     * 云供应商名称
     */
    @IgnoreAll
    private String vendorName;
    /**
     * 是否已解决
     */
    @IgnoreAll
    private Boolean solved;
    /**
     * 是否已确认
     */
    @IgnoreAll
    private Boolean confirmed;
    /**
     * 持续时长
     */
    @IgnoreAll
    private String duration;

    @Column("attach_user_group")
    private String attachUserGroup;// 所属用户

    public Boolean isSolved() {
        return this.gmtSolve != null;
    }

    public Boolean isConfirmed() {
        return this.gmtConfirm != null;
    }

    public Boolean getSolved() {
        return this.solver != null && this.gmtSolve != null;
    }

    public Boolean getConfirmed() {
        return this.confirmer != null && this.gmtConfirm != null;
    }

    /**
     * 根据产生时间计算告警持续时长
     *
     * @return
     */
    public String getDuration() {
        if (this.gmtTrigger != null) {
            Date end = null;
            if (this.isSolved()) {
                end = this.getGmtSolve();
            } else {
                end = new Date();
            }
            long different = end.getTime() - this.gmtTrigger.getTime();
            long secondsInMilli = 1000;
            long minutesInMilli = secondsInMilli * 60;
            long hoursInMilli = minutesInMilli * 60;
            long daysInMilli = hoursInMilli * 24;
            long elapsedDays = different / daysInMilli;
            different = different % daysInMilli;
            long elapsedHours = different / hoursInMilli;
            different = different % hoursInMilli;
            long elapsedMinutes = different / minutesInMilli;
            different = different % minutesInMilli;
            long elapsedSeconds = different / secondsInMilli;
            String str = String.format("%d天%d小时%d分钟%d秒", elapsedDays, elapsedHours, elapsedMinutes, elapsedSeconds);
            if (elapsedDays == 0 && elapsedHours == 0 && elapsedMinutes == 0) {
                str = str.replace("0天0小时0分钟", "");
            }
            if (elapsedDays == 0 && elapsedHours == 0) {
                str = str.replace("0天0小时", "");
            }
            if (elapsedDays == 0) {
                str = str.replace("0天", "");
            }
            return str;
        } else {
            return null;
        }
    }

    public void setTarget(Long target) {
        this.target = String.valueOf(target);
    }

    public void setTarget(String target) {
        this.target = target;
    }

}