package com.bocloud.cms.entity;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreUpdate;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 告警规则绑定资源
 *
 * @author fanzhenxi
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@EqualsAndHashCode(callSuper = false)
@Data
@NoArgsConstructor
@Table("monitor_resource_relation")
public class ResourceRelation extends Generic {

    /**
     * 主键
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;

    /**
     * 规则组ID
     */
    @Column("group_id")
    private Long groupId;

    /**
     * 资源标识
     */
    @Column("target")
    private String target;

    /**
     * 创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    /**
     * 修改时间
     */
    @Column("gmt_modify")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;

    /**
     * 是否已删除
     */
    @Column("is_deleted")
    private Boolean deleted;

    public ResourceRelation(Long groupId, String target) {
        this.groupId = groupId;
        this.target = target;
    }

    public ResourceRelation(Long groupId, String target, Date gmtCreate, Boolean deleted) {
        this.groupId = groupId;
        this.target = target;
        this.gmtCreate = gmtCreate;
        this.deleted = deleted;
    }

}

