package com.bocloud.cms.entity;

import com.bocloud.cms.model.OpenStackServiceModel;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 告警规则实体类
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Data
@EqualsAndHashCode(callSuper = false)
@AllArgsConstructor
@NoArgsConstructor
@Table("monitor_rule_group")
public class RuleGroup extends Generic {

    /**
     * id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;

    /**
     * 阈值名称
     */
    @Column("name")
    private String name;

    /**
     * 资源类型
     */
    @IgnoreUpdate
    @Column("resource_type")
    private String resourceType;

    /**
     * 分发策略ID
     */
    @Column("policy_id")
    private Long policyId;

    /**
     * 创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    /**
     * 修改时间
     */
    @Column("gmt_modify")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;

    /**
     * 创建者id
     */
    @IgnoreUpdate
    @Column("creator_id")
    private Long creatorId;

    /**
     * 租戶ID
     */
    @IgnoreUpdate
    @Column("tenant_id")
    private Long tenantId;

    /**
     * 修改者id
     */
    @Column("mender_id")
    private Long menderId;

    /**
     * 是否已删除
     */
    @Column("is_deleted")
    private Boolean deleted;

    /**
     * 是否已删除
     */
    @Column("is_global")
    private Boolean isGlobal;

    /**
     * 是否开启告警
     */
    @Column("enable")
    private Boolean enable;

    /**
     * 描述
     */
    @Column("remark")
    private String remark;

    @Column("creator_catalog")
    private String creatorcatalog;

    @Column("mender_catalog")
    private String menderCatalog;


    /**
     * 资源关联id
     * resourceId-->target
     */
    @IgnoreAll
    private List<String> resourceIds;

    /**
     * 告警规则
     * 一个告警中可能会有多个告警规则
     */
    @IgnoreAll
    private List<Rule> rules;

    /**
     * 告警分发策略对象
     */
    @IgnoreAll
    private String policyName;

    /**
     * 获取创建者名称
     */
    @IgnoreAll
    private String createName;

    /**
     * openStack服务 回显特殊处理
     */
    @IgnoreAll
    private List<OpenStackServiceModel> openStackServiceModels;

}
