package com.bocloud.ims.entity;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.DateSerializer;
import com.megatron.common.utils.DaySerializer;
import com.megatron.entity.bean.GenericEntity;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 云主机实体类
 *
 * @author DZG
 * @since V1.0 2016年9月12日
 */
@Data
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
public class CloudServer extends GenericEntity {

    /**
     * ID
     */
    private Long id;
    /**
     * 账号
     */
    private String account;
    /**
     * 密码
     */
    private String password;
    /**
     * 密钥名称
     */
    private String keypairName;
    /**
     * CPU核数
     */
    private Integer cpu;
    /**
     * CPU核数
     */
    private Double hmcCpu;
    /**
     * 内存大小
     */
    private BigDecimal memory;
    /**
     * 磁盘容量
     */
    private Integer disk;
    /**
     * 私有IP
     */
    private String privateIps;
    /**
     * 公有IP
     */
    private String publicIps;
    /**
     * 管理端口
     */
    private Integer managerPort;
    /**
     * 云供应商Id
     */
    private Long vendorId;
    private Long env;
    private String vendorType;
    /**
     * 供应商区域ID
     */
    private String region;
    /**
     * 镜像ID
     */
    private String imageUuid;
    /**
     * 配置ID
     */
    private String flavorUuid;
    /**
     * 网络ID
     */
    private String networkUuid;
    /**
     * 云主机ID
     */
    private String instanceId;
    /**
     * UUID
     */
    private String uuid;
    private String subnetUuid;
    /**
     * 区域
     */
    private String zone;
    /**
     * 路由ID
     */
    private String routerUuid;
    private String managerIp;
    /**
     * 是否是模板
     */
    private Boolean isTemplate;
    /**
     * 区域ID
     */
    private String regionId;
    /**
     * 网络计费类型
     */
    private String internetChargeType;
    /**
     * 公网入带宽最大值
     */
    private Integer internetMaxBandwidthIn;
    /**
     * 公网出带宽最大值
     */
    private Integer internetMaxBandwidthOut;
    /**
     * 实例付费类型
     */
    private String instanceChargeType;
    /**
     * 实例的网络类型
     */
    private String instanceNetworkType;
    /**
     * 实例类型
     */
    private String instanceType;
    /**
     * 系统盘的类型
     */
    private String systemDiskCategory;
    /**
     * 系统盘的大小
     */
    private Integer systemDiskSize;
    /**
     * 数据盘
     */
    private String dataDisk;
    /**
     * 宿主机id
     */
    private Long hostId;
    private Long projectId;
    /**
     * 数据中心id
     */
    private Long vdcId;
    /**
     * 模板id
     */
    private Long templateId;
    /**
     * 系统类型
     */
    private String osCategory;
    /**
     * 系统名称
     */
    private String osName;
    /**
     * 系统版本
     */
    private String osVersion;
    /**
     * 资源分组
     */
    private String resourceGroup;
    private Boolean toolsInstalled;
    private Boolean toolsRunning;
    private Boolean toolsExpired;
    private String hostName;
    private String datastores;
    private String resourceId;
    private String resourceLabel;
    /**
     * vm的创建时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtVmCreate;
    private Boolean recycle;
    /**
     * 放入回收站时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date recycleTime;
    private String cpuModel;
    private String modelName;
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtVmremove;
    /**
     * 租户ID
     */
    private Long tenantId;
    /**
     * 是否开启CPU热插拔
     */
    private Boolean cpuHotEnable;
    /**
     * 是否开启内存热插拔
     */
    private Boolean memHotEnable;
    /**
     * 资源池ID
     */
    private Long resPoolId;
    /**
     * 全局唯一标识
     */
    private String guid;
    /**
     * VMware Tools配置
     */
    private String toolsConfig;
    private String securityGroupId;
    /**
     * azure 资源组名称
     */
    private String azureResourceGroupName;
    private String vpcUuid;

    /**
     * azure 网络接口
     * azure 新增字段.start
     */
    private String networkInterfaceUuid;
    /**
     * 账号类别
     */
    private Catalog catalog;
    /**
     * sku id
     */
    private Long skuId;
    /**
     * 类型id
     */
    private Long categoryId;
    /**
     * 内网带宽 json格式保存
     */
    private String internetBandwidth;
    /**
     * openstack底层租户id
     */
    private String tenantUuid;
    /**
     * 资源组ID
     */
    private String resourceGroupUuid;
    private Long availabilitySetId;
    /**
     * 备份数量
     */
    private String backupCount;
    private String vaultsUuid;
    /**
     * 堡垒机资源ids的json字符串
     */
    private String fortress;
    /**
     * 宿主机名称
     */
    private String hostServerName;
    private Integer extendDisk;
    /**
     * 过期时间
     */
    @JsonSerialize(using = DaySerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expiredTime;
    /**
     * 云供应商名称
     */
    private String vendorName;
    /**
     * 供应商区域名称
     */
    private String regionName;
    private String resourceGroupName;
    /**
     * 模板名称
     */
    private String template;
    /**
     * 模板UUID
     */
    private String templateUuid;
    private String datastore;
    /**
     * 快照数量
     */
    private Integer snapshotCount;
    private String clusterName;
    private Long clusterId;
    private String vdcName;
    private List<String> stores;
    /**
     * 绑定的浮动IP的ID
     */
    private Long fipId;
    /**
     * 云主机绑定的浮动IP
     */
    private String floatingIp;
    /**
     * 安全组
     */
    private String groups;
    /**
     * 存储ID
     */
    private Long volumeId;
    /**
     * 根磁盘容量
     */
    private Integer rootSize;
    /**
     * 所属租户
     */
    private String tenantName;
    /**
     * 绑定的所有安全组
     */
    private String sgroupNames;
    private String dns;
    private String gateway;
    /**
     * 是否从卷创建
     */
    private Boolean bootFromVolume;
    /**
     * 所属资源池value
     */
    private String resPoolVal;
    private String host;
    private String vmHostName;
    private String projectName;
    private String projectVisibility;
    private String targetName;
    private String floatingIpName;
    private List<String> isos;
    private List<String> securityGroups;
    private String vpcId;
    private String vSwitchId;
    private List<String> vSwitchIds;
    private String zoneName;
    private List<Integer> disks;
    private String flavorName;
    private String imageName;
    private List<Long> labelIds;
    private String labelNames;
    private Integer totalDisks;
    /**
     * 类别
     */
    private String category;
    /**
     * 资源类型{value: PHYSICAL,LOGICAL}
     */
    private String type;
    /**
     * 内存使用率
     */
    private String memUsage;
    /**
     * cpu使用率
     */
    private String cpuUsage;
    /**
     * cpu总量
     */
    private String cpuTotal;
    /**
     * 内存总量
     */
    private String memTotal;
    /**
     * 通断状态
     */
    private String onOffStatus;
    /**
     * 响应时长
     */
    private String responseTime;
    /**
     * 监控IP
     */
    private String monitorIp;
    /**
     * 是否存在定时快照任务
     */
    private Boolean hasSnapshotTask;
    /**
     * 系统盘uuid
     */
    private String systemDiskUuid;
    /**
     * 内网带宽
     */
    private Integer bandwidth;
    /**
     * openstack底层租户名称
     */
    private String osTenantName;
    /**
     * 用户名称
     */
    private String userName;
    /**
     * 业务
     */
    private String businessName;
    /**
     * vcenter磁盘格式(sameAsSource与源格式相同|flat厚置备延迟置零|thick厚置备快速置零|thin精简置备)
     */
    private String diskType;
    /**
     * biosUuid
     */
    private String biosUuid;


    private String attachUserGroup;
    /**
     * 运行时长
     */
    private String statusContinueTime;

    private String localId;

    private List<String> subnetIds;

    private List<String> volumeIds;


}
