package com.bocloud.cms.repository;

import com.bocloud.cms.entity.MonitorAllocationRatio;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 超分比
 *
 * @author zhangyf
 * @version 5.5.0
 * @date 2022/2/21 18:01
 */
@Repository
public class AllocationRatioRepository extends BasicGenericDao<MonitorAllocationRatio, Long> {

    public AllocationRatioRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 按云平台ID查询超分比
     *
     * @param vendorId 云平台ID
     * @return 云平台超分比
     */
    public MonitorAllocationRatio query(Long vendorId) {
        String sql = "select * from monitor_allocation_ratio where vendor_id = :vendorId ";
        Map<String, Object> paramMap = MapTools.simpleMap("vendorId", vendorId);
        List<MonitorAllocationRatio> ratios = this.list(MonitorAllocationRatio.class, sql, paramMap);
        return ListTool.isEmpty(ratios) ? null : ratios.get(0);
    }

    /**
     * 按云平台ID查询超分比配置
     *
     * @param vendorId 云平台ID
     */
    public void delete(Long vendorId) {
        String sql = "delete from monitor_allocation_ratio where vendor_id = :vendorId ";
        Map<String, Object> paramMap = MapTools.simpleMap("vendorId", vendorId);
        this.execute(sql, paramMap);
    }

}
