package com.bocloud.cms.repository;

import com.bocloud.cms.entity.MonitorAnalysisRule;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 监控分析监控分析Dao层实现类的处理
 *
 * @author zhangyf
 * @version 5.5.0
 * @date 2022/2/21 18:01
 */
@Repository
public class AnalysisRuleRepository extends BasicGenericDao<MonitorAnalysisRule, Long> {

    public AnalysisRuleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 监控分析规则总数
     *
     * @param params 参数集合
     * @return 总数
     */
    public int count(List<Param> params) {
        String sql = "select count(1) from monitor_analysis_rule a where is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, null).intValue();
    }

    /**
     * 监控分析规则列表
     *
     * @param pager 分页
     * @return 监控分析规则列表
     */
    public List<MonitorAnalysisRule> list(Pager pager) {
        String sql = "select * from monitor_analysis_rule where is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "");
        return this.list(MonitorAnalysisRule.class, sql);
    }

    /**
     * 按ID查询监控分析规则详情
     *
     * @param id ID
     * @return 监控分析规则详情
     */
    public MonitorAnalysisRule query(Long id) {
        String sql = "select * from monitor_analysis_rule where is_deleted = 0 and id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<MonitorAnalysisRule> rules = this.list(MonitorAnalysisRule.class, sql, paramMap);
        return ListTool.isEmpty(rules) ? null : rules.get(0);
    }

    /**
     * 校验名称是否存在
     *
     * @param id   ID
     * @param name 名称
     * @return 是否存在
     */
    public boolean nameExist(Long id, String name) {
        String sql = "select * from monitor_analysis_rule where is_deleted = 0 and name = :name ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        if (null != id) {
            sql += "and id <> :id ";
            paramMap.put("id", id);
        }
        List<MonitorAnalysisRule> rules = this.list(MonitorAnalysisRule.class, sql, paramMap);
        return !rules.isEmpty();
    }

}
