package com.bocloud.cms.repository;

import com.bocloud.cms.entity.AnalysisSnapshot;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 监控分析快照
 *
 * @author zhangyf
 * @since 2019-08-15
 */
@Repository
public class AnalysisSnapshotRepository extends BasicGenericDao<AnalysisSnapshot, Long> {

    public AnalysisSnapshotRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 获取总数
     *
     * @param params 参数集合
     * @return 总数
     */
    public int count(List<Param> params) {
        String sql = "select count(1) from monitor_analysis_snapshot a where is_deleted = 0   ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 监控分析快照列表
     *
     * @param pager 分页
     * @return 监控分析快照列表
     */
    public List<AnalysisSnapshot> list(Pager pager) {
        String sql = "select s.*,r.name as rule_name from monitor_analysis_snapshot s left join monitor_analysis_rule r on s.rule_id = r.id where s.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "s");
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        return this.list(AnalysisSnapshot.class, sql, paramMap);
    }

    /**
     * 监控分析快照详情
     *
     * @param id ID
     * @return 监控分析快照列表
     */
    public AnalysisSnapshot query(Long id) {
        String sql = "select * from monitor_analysis_snapshot where is_deleted = 0 and id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<AnalysisSnapshot> snapshots = this.list(AnalysisSnapshot.class, sql, paramMap);
        return ListTool.isEmpty(snapshots) ? null : snapshots.get(0);
    }

}
