package com.bocloud.cms.repository;

import com.bocloud.cms.entity.MonitorHostIpmi;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 宿主机IPMI监控
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Repository
public class MonitorHostIpmiRepository extends BasicGenericDao<MonitorHostIpmi, Long> {

    public MonitorHostIpmiRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 按宿主机ID查询
     *
     * @param hostId 宿主机ID
     * @return 宿主机IPMI信息
     */
    public MonitorHostIpmi query(Long hostId) {
        String sql = "select * from monitor_host_ipmi where is_deleted = 0 and host_id = :hostId ";
        Map<String, Object> paramMap = MapTools.simpleMap("hostId", hostId);
        List<MonitorHostIpmi> ipRelations = this.list(MonitorHostIpmi.class, sql, paramMap);
        return ListTool.isEmpty(ipRelations) ? null : ipRelations.get(0);
    }

    /**
     * 宿主机IPMI信息集合
     *
     * @return 宿主机IPMI信息集合
     */
    public List<MonitorHostIpmi> list() {
        String sql = "select * from monitor_host_ipmi where is_deleted = 0 ";
        return this.list(MonitorHostIpmi.class, sql);
    }
}
