package com.bocloud.cms.repository;

import com.bocloud.cms.entity.MonitorResourceType;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 告警配置资源类型
 *
 * @author fanzhenxi
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Repository
public class MonitorResourceTypeRepository extends BasicGenericDao<MonitorResourceType, Long> {

    public MonitorResourceTypeRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 查询告警配置资源总数
     *
     * @param params 参数集合
     * @return 数量
     */
    public int count(List<Param> params) {
        String sql = "select count(1) from monitor_resource_type where is_deleted = 0 ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 查询告警配置资源列表
     *
     * @param pager 分页
     * @return 资源类型列表
     */
    public List<MonitorResourceType> list(Pager pager) {
        String sql = "select * from monitor_resource_type where is_deleted = 0 ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        sql = getQueryBuilder().buildRaw(sql, pager, "");
        return this.list(MonitorResourceType.class, sql, paramMap);
    }

    /**
     * 根据资源名称去进行查询 名称为枚举类
     *
     * @param name    资源名称
     * @param context 当前用户上下文
     * @return
     */
    public MonitorResourceType getByName(String name, RequestContext context) {
        String sql = "select * from monitor_resource_type where is_deleted = 0 and name = :name";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        List<MonitorResourceType> plugins = this.list(MonitorResourceType.class, sql, paramMap);
        return ListTool.isEmpty(plugins) ? null : plugins.get(0);
    }

    /**
     * 根据资源名称去进行查询
     *
     * @param resourceType 监控资源类型
     * @param context      当前用户上下文
     * @return
     */
    public MonitorResourceType getByResourceType(String resourceType, RequestContext context) {
        String sql = "select * from monitor_resource_type where is_deleted = 0 and resource_type = :resourceType";
        Map<String, Object> paramMap = MapTools.simpleMap("resourceType", resourceType);
        List<MonitorResourceType> plugins = this.list(MonitorResourceType.class, sql, paramMap);
        return ListTool.isEmpty(plugins) ? null : plugins.get(0);
    }

    /**
     * 根据监控资源名称判断是否存在
     *
     * @param name    监控资源名称
     * @param context 当前用户上下文
     * @return
     */
    public boolean existsSameName(String name, RequestContext context) {
        String sql = "select count(1) from monitor_resource_type where name = :name and is_deleted = 0 ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        return this.countQuery(sql, paramMap).intValue() > 0;
    }

    /**
     * 根据监控资源类型判断是否存在
     *
     * @param resourceType 监控资源类型
     * @param context      当前用户上下文
     * @return
     */
    public boolean existsSameResourceType(String resourceType, RequestContext context) {
        String sql = "select count(1) from monitor_resource_type where resource_type = :resourceType and is_deleted = 0 ";
        Map<String, Object> paramMap = MapTools.simpleMap("resourceType", resourceType);
        return this.countQuery(sql, paramMap).intValue() > 0;
    }
}
