package com.bocloud.cms.intf;

import com.bocloud.cms.entity.MonitorHostIpmi;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

/**
 * 宿主机
 *
 * @author zhangyf
 * @since 2019-12-23
 */
public interface HostService {

    /**
     * 宿主机资源概览
     *
     * @param id   宿主机ID
     * @param type 类型
     * @return 宿主机资源概览信息
     */
    GeneralResult<?> hostOverview(Long id, String type);

    /**
     * 宿主机Cpu、内存、磁盘信息
     *
     * @param id   宿主机ID
     * @param type 类型
     * @return 宿主机Cpu、内存、磁盘信息
     */
    GeneralResult<?> hostInfo(Long id, String type);

    /**
     * 主机列表，先从cmp查询，然后再查询ruleRelation，最后将结果拼到结果集里
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 主机列表
     */
    GeneralResult<?> list(Pager pager, RequestContext context);

    /**
     * 主机绑定ipmi信息
     *
     * @param monitorHostIpmi Impi绑定信息
     * @param context         当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> binding(MonitorHostIpmi monitorHostIpmi, RequestContext context);

    /**
     * 删除宿主机ipmi信息
     *
     * @param hostId 宿主机ID
     * @param userId 用户ID
     * @return 操作结果
     */
    GeneralResult<?> unbind(Long hostId, Long userId);

    /**
     * 查询ipmi详情
     *
     * @param hostId 宿主机ID
     * @return 详情
     */
    GeneralResult<?> detail(Long hostId);

}