package com.bocloud.cms.intf;

import com.bocloud.cms.entity.Metric;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

import java.util.List;

/**
 * 监控指标
 *
 * @author fanzhenxi
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
public interface MetricService {
    /**
     * 指标列表展示
     *
     * @param resourceType 资源类型
     * @return 指标列表
     */
    GeneralResult<?> list(String resourceType);

    /**
     * 按id查询，先从缓存中查询，如果未命中，则查询数据库
     *
     * @param id 监控指标ID
     * @return 监控指标
     */
    Metric query(Long id);

    /**
     * 查询资源监控指标列表
     *
     * @param pager 分页
     * @return 监控指标列表
     */
    GeneralResult<?> monitorMetricList(Pager pager);

    /**
     * 创建监控指标
     *
     * @param metric  监控指标
     * @param context 当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> create(Metric metric, RequestContext context);

    /**
     * 修改监控指标
     *
     * @param metric  监控指标
     * @param context 当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> modify(Metric metric, RequestContext context);

    /**
     * 删除监控指标
     *
     * @param id      监控指标ID
     * @param context 当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> remove(Long id, RequestContext context);

    /**
     * 批量删除监控指标
     *
     * @param ids     监控指标ID集合
     * @param context 当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> batchRemove(List<Long> ids, RequestContext context);

    /**
     * 获取该资源类型下所有status为开启的指标列表
     *
     * @param resourceType 资源类型
     * @return 监控指标列表集合
     */
    GeneralResult<?> getByResourceType(String resourceType);
}
