package com.bocloud.cms.intf.external;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.model.LineChartResult;

import java.util.List;
import java.util.Map;

/**
 * ManageOne外部接口
 *
 * @author zhangyf
 * @since 2020-07-02
 */
public interface RemoteManageOneService {

    /**
     * 资源详情
     *
     * @param vendorId  云平台ID
     * @param className className
     * @param id        ID
     * @return 资源详情
     */
    JSONObject getInstanceDetail(Long vendorId, String className, String id);

    /**
     * 查询指定类型的资源
     *
     * @param vendorId        云平台ID
     * @param className       className
     * @param paramMap        参数集合
     * @param contentSelector contentSelector
     * @return 指定类型的资源
     */
    List<JSONObject> listInstance(Long vendorId, String className, Map<String, Object> paramMap, List<String> contentSelector);

    /**
     * 查询资源池的云主机、物理机状态数目统计
     *
     * @param vendorId 云平台ID
     * @param id       ID
     * @return 资源池的云主机、物理机状态数目统计
     */
    JSONObject resourcePoolStatistics(Long vendorId, String id);

    /**
     * 查询可用区的云主机、物理机状态数目统计
     *
     * @param vendorId 云平台ID
     * @param id       ID
     * @return 可用区的云主机、物理机状态数目统计
     */
    JSONObject azoneStatistics(Long vendorId, String id);

    /**
     * 资源池的当前容量
     *
     * @param vendorId 云平台ID
     * @param id       ID
     * @return 资源池的当前容量
     */
    JSONObject resourcePoolCurrentCapacities(Long vendorId, String id);

    /**
     * 可用区的当前容量
     *
     * @param vendorId 云平台ID
     * @param id       ID
     * @return 可用区的当前容量
     */
    JSONObject azoneCurrentCapacities(Long vendorId, String id);

    /**
     * 查询资源池存储池的容量
     *
     * @param vendorId 云平台ID
     * @param id       ID
     * @return 资源池存储池的容量
     */
    LineChartResult resourcePoolStoragePoolCapacities(Long vendorId, String id);

    /**
     * 查询可用区存储池的容量
     *
     * @param vendorId 云平台ID
     * @param id       ID
     * @return 可用区存储池的容量
     */
    LineChartResult azoneStoragePoolCapacities(Long vendorId, String id);
}
