package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.MonitoringAble;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.bocloud.ims.entity.CloudServer;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zhangyf
 */
@Service
@Slf4j
public class BaiDuServiceImpl implements MonitoringAble<CloudServer>, InitializingBean {

    private final CmpCloudServerService cmpCloudServerService;
    private final ResourceAlarmStatusService resourceAlarmStatusService;
    private final AlarmService alarmService;

    public BaiDuServiceImpl(CmpCloudServerService cmpCloudServerService, ResourceAlarmStatusService resourceAlarmStatusService,
                            AlarmService alarmService) {
        this.cmpCloudServerService = cmpCloudServerService;
        this.resourceAlarmStatusService = resourceAlarmStatusService;
        this.alarmService = alarmService;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_BaiDu_VM, this);
    }

    @Override
    public List<CloudServer> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetIds).substring(1, JSONObject.toJSONString(targetIds).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = cmpCloudServerService.list(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(CloudServer targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>();
        params.put("instance_id", targetObject.getInstanceId());
        return params;
    }

    @Override
    public CloudServer parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String instanceId = String.valueOf(alert.getLabels().get("instance_id"));
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
        GeneralResult result = cmpCloudServerService.list(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
        Assert.isTrue(list != null && !list.isEmpty(), "vm not found,instanceId:" + instanceId);
        return list.get(0);
    }

    @Override
    public void destroy(CloudServer cloudServer, String catalog) throws Exception {
        log.info("destroy baidu vm : " + cloudServer.getId());
        alarmService.solveByCloudServer(cloudServer.getId(), catalog, 1L, "系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, CloudServer object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getPrivateIps() == null ? object.getName() : object.getPrivateIps();
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, CloudServer object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }
}
