package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.utils.DateUtils;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.enums.Operator;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.FusionComputeService;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.bocloud.ims.entity.CloudServer;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;


/**
 * @author wpj
 * @since 2022/8/18
 */
@Service
@Slf4j
public class FusionComputeServiceImpl implements FusionComputeService, InitializingBean {
    @Autowired
    private CmpCloudServerService cmpCloudServerService;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private AlarmService alarmService;

    /**
     * 查询所有FusionCompute虚拟机列表信息
     *
     * @param pager 分页
     * @param context 当前用户上下文
     * @param request
     * @return
     */
    @Override
    public GeneralResult listOfVm(Pager pager, RequestContext context, HttpServletRequest request) {
        GeneralResult result = cmpCloudServerService.list(pager, request.getHeader(Common.TOKEN));
        JSONObject data = (JSONObject) result.getData();
        List<CloudServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServerModel.class);
        List<String> instanceIdList = new ArrayList<>();
        for (CloudServerModel cloudServer : list) {
            instanceIdList.add(cloudServer.getInstanceId());
        }
        String instanceIds = StringUtils.join(instanceIdList.toArray(), "|");

        List<LabelParam> params = Collections.singletonList(new LabelParam("urn", instanceIds, Operator.RE_EQ));

        String cpuExpr = "round(fusioncompute_vm_cpu_usage{},0.01)";
        String memoryExpr = "round(fusioncompute_vm_mem_usage{},0.01)";
        String statusContinueTime = "round(fusioncompute_vm_status_continue_time{},0.01)";

        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);
        List<PrometheusResult> statusTimeList = prometheusService.query(statusContinueTime, params);

        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HWACCESS_VM.name(), targets, context);
        for (CloudServerModel cloudServer : list) {
            /*ims适配 暂时不做处理
            if (cloudServer.getStatusContinueTime()!=null){
                String resultTime = DateUtils.dateConvert(Double.parseDouble(cloudServer.getStatusContinueTime()));
                cloudServer.setStatusContinueTime(resultTime);
            }*/
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(cloudServer.getId().toString()))
                    .findAny().isPresent();
            cloudServer.setAlarmEnable(alarmEnable);

            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("urn").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("urn").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                }
            }
            for (PrometheusResult statusTime : statusTimeList) {
                if (statusTime.getMetric().get("urn").equals(cloudServer.getInstanceId())) {
                    Double value = Double.valueOf((String) statusTime.getValue()[1]);
                    String promeResultTime = DateUtils.dateConvert(value);
                    cloudServer.setStatusContinueTime(promeResultTime);
                }
            }
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<CloudServer> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        try {
            Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetIds).substring(1, JSONObject.toJSONString(targetIds).length() - 1)), Sign.IN);
            List<Param> params = Collections.singletonList(param);
            GeneralResult result = cmpCloudServerService.list(new Pager(1, Integer.MAX_VALUE, params, null), null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
        } catch (Exception e) {
            log.error("获取fc列表失败！" + e);
            return null;
        }
    }


    @Override
    public Map<String, String> getPrometheusExprParam(CloudServer cloudServer, Metric metric) {
        Map<String, String> params = new HashMap<>();
        if (MonitorResourceType.MONITOR_HWACCESS_VM.name().equals(metric.getResourceType())) {//虚拟机
            params.put("urn", cloudServer.getInstanceId());
            params.put("vendor_id", String.valueOf(cloudServer.getVendorId()));
        }
        return params;
    }

    @Override
    public CloudServer parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        if (MonitorResourceType.MONITOR_HWACCESS_VM.name().equals(metric.getResourceType())) {
            String instanceId = String.valueOf(alert.getLabels().get("urn"));
            List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
            GeneralResult result = cmpCloudServerService.list(new Pager(1, 1, params, null), null);
            // Assert.isTrue(result.isSuccess(), result.getMessage());
            if (result.isFailed()) {
                log.warn("告警虚拟机：{}, 不存在", instanceId);
                return null;
            }
            JSONObject data = (JSONObject) result.getData();
            List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
            // Assert.isTrue(list != null && !list.isEmpty(), "vm not found,instanceId:" + instanceId);
            if (list == null || list.isEmpty()) {
                log.warn("告警虚拟机：{}, 不存在", instanceId);
                return null;
            }
            return list.get(0);
        }
        return null;
    }


    @Override
    public void destroy(CloudServer cloudServer, String catalog) throws Exception {
        log.info("destroy fc vm : " + cloudServer.getId());
        alarmService.solveByCloudServer(cloudServer.getId(), catalog, 1L, "系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, CloudServer cloudServer) {
        alarm.setTarget(cloudServer.getId());
        alarm.setVendorId(cloudServer.getVendorId());
        alarm.setOwnerId(cloudServer.getCreatorId());
        // 修改来源
        alarm.setTargetName(cloudServer.getName() + "(" + cloudServer.getPublicIps() + ")");
        // 华为云桌面: 添加站点、所属用户
        alarm.setAttachUserGroup(cloudServer.getAttachUserGroup());
        alarm.setVendorName(cloudServer.getVendorName());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, CloudServer cloudServer) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(cloudServer.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_HWACCESS_VM, this);
    }
}
