package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.HmcPartitionModel;
import com.bocloud.cms.model.HmcServerModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.hmc.HmcService;
import com.bocloud.ims.entity.hmc.HmcPartition;
import com.bocloud.ims.entity.hmc.HmcServer;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;

/**
 * hmc实现层
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
@Slf4j
public class HmcServiceImpl implements com.bocloud.cms.intf.HmcService, InitializingBean {


    @Autowired
    private HmcService hmcService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public GeneralResult listOfServer(Pager pager, RequestContext context) {

        GeneralResult result = hmcService.listOfServer(pager);
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<HmcServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(HmcServerModel.class);
        if (list.isEmpty()) {
            data.put(Common.ROWS, list);
            return result;
        }
        List<String> targets = new ArrayList<>();
        for (HmcServerModel model : list) {
            targets.add(model.getId().toString());
        }

        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HMC_SERVER.name(), targets, context);
        for (HmcServerModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public GeneralResult listOfPartition(Pager pager, RequestContext context) {

        GeneralResult result = hmcService.listOfPartition(pager);
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<HmcPartitionModel> list = data.getJSONArray(Common.ROWS).toJavaList(HmcPartitionModel.class);
        if (list.isEmpty()) {
            data.put(Common.ROWS, list);
            return result;
        }
        List<String> targets = new ArrayList<>();
        for (HmcPartitionModel model : list) {
            targets.add(model.getId().toString());
        }
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HMC_VIOS.name(), targets, context);
        for (HmcPartitionModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List listTargetObjects(List targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        for (Object targetId : targetIds) {
            targetList.add(Integer.valueOf(targetId.toString()));
        }
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        if (MonitorResourceType.MONITOR_HMC_SERVER.equals(resourceType)) {
            List<Param> params = Collections.singletonList(param);
            GeneralResult result = hmcService.listOfServer(new Pager(1, Integer.MAX_VALUE, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(HmcServer.class);
        }
        if (MonitorResourceType.MONITOR_HMC_VIOS.equals(resourceType)
                || MonitorResourceType.MONITOR_HMC_AIX.equals(resourceType)) {
            List<Param> params = Collections.singletonList(param);
            GeneralResult result = hmcService.listOfServer(new Pager(1, Integer.MAX_VALUE, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(HmcPartition.class);
        }
        return null;
    }

    @Override
    public Map<String, String> getPrometheusExprParam(Object targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        // Hmc主机
        if (MonitorResourceType.MONITOR_HMC_SERVER.name().equals(metric.getResourceType())) {
            HmcServer hmcServer = (HmcServer) targetObject;
            if (metric.getExpr().contains("share_cpu_pool")) {
                params.put("parentUuid", hmcServer.getUuid());
            } else {
                params.put("instanceUuid", hmcServer.getUuid());
            }
            // Hmc分区
        } else if (MonitorResourceType.MONITOR_HMC_AIX.name().equals(metric.getResourceType()) ||
                MonitorResourceType.MONITOR_HMC_VIOS.name().equals(metric.getResourceType())) {
            HmcPartition hmcPartition = (HmcPartition) targetObject;
            if (metric.getExpr().endsWith("_rb") || metric.getExpr().endsWith("_sb") || metric.getExpr().endsWith("_wb")) {
                params.put("parentUuid", hmcPartition.getUuid());
            } else {
                params.put("instanceUuid", hmcPartition.getUuid());
            }
        }
        return params;
    }

    @Override
    public Object parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        if (MonitorResourceType.MONITOR_HMC_SERVER.name().equals(metric.getResourceType())) {
            List<Param> params;
            String uuid;
            if (metric.getExpr().contains("share_cpu_pool")) {
                uuid = String.valueOf(alert.getLabels().get("parentUuid"));
            } else {
                uuid = String.valueOf(alert.getLabels().get("instanceUuid"));
            }
            params = Collections.singletonList(new Param(MapTools.simpleMap("uuid", uuid), Sign.EQ));
            GeneralResult result = hmcService.listOfServer(new Pager(1, 1, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<HmcServer> list = data.getJSONArray(Common.ROWS).toJavaList(HmcServer.class);
            Assert.isTrue(list != null && !list.isEmpty(), "vm not found,uuid:" + uuid);
            return list.get(0);
        } else if (MonitorResourceType.MONITOR_HMC_AIX.name().equals(metric.getResourceType()) ||
                MonitorResourceType.MONITOR_HMC_VIOS.name().equals(metric.getResourceType())) {
            // Hmc分区
            List<Param> params;
            String uuid;
            if (metric.getExpr().endsWith("_rb") || metric.getExpr().endsWith("_sb") || metric.getExpr().endsWith("_wb")) {
                uuid = String.valueOf(alert.getLabels().get("parentUuid"));
            } else {
                uuid = String.valueOf(alert.getLabels().get("instanceUuid"));
            }
            params = Collections.singletonList(new Param(MapTools.simpleMap("uuid", uuid), Sign.EQ));
            GeneralResult result = hmcService.listOfServer(new Pager(1, 1, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<HmcPartition> list = data.getJSONArray(Common.ROWS).toJavaList(HmcPartition.class);
            Assert.isTrue(list != null && !list.isEmpty(), "vm not found,uuid:" + uuid);
            return list.get(0);
        }
        return null;
    }

    @Override
    public void destroy(Object object, String catalog) throws Exception {
        if (MonitorResourceType.MONITOR_HMC_SERVER.name().equals(catalog)) {
            HmcServer hmcServer = (HmcServer) object;
            log.info("hmc service : " + hmcServer.getId());
            //TODO
        } else {
            HmcPartition hmcPartition = (HmcPartition) object;
            log.info("hmc partition : " + hmcPartition.getId());
            //TODO
        }
    }

    @Override
    public void wrapAlarm(Alarm alarm, Object object) {
        if (MonitorResourceType.MONITOR_HMC_SERVER.name().equals(alarm.getType())) {
            HmcServer hmcServer = (HmcServer) object;
            alarm.setTarget(hmcServer.getId());
            if (hmcServer.getName()!=null & hmcServer.getPrimaryIpAddress()!=null){
                alarm.setTargetName(hmcServer.getName() + "(" + hmcServer.getPrimaryIpAddress() +")");
            }else{
                String targetName = hmcServer.getPrimaryIpAddress() == null ? hmcServer.getName() : hmcServer.getPrimaryIpAddress();
                alarm.setTargetName(targetName);
            }
            alarm.setVendorId(hmcServer.getVendorId());
        } else if (MonitorResourceType.MONITOR_HMC_VIOS.name().equals(alarm.getType()) ||
                MonitorResourceType.MONITOR_HMC_AIX.name().equals(alarm.getType())) {
            HmcPartition hmcPartition = (HmcPartition) object;
            alarm.setTarget(hmcPartition.getId());
            if (hmcPartition.getName()!=null & hmcPartition.getIp()!=null){
                alarm.setTargetName(hmcPartition.getName() + "(" + hmcPartition.getIp() +")");
            }else{
                String targetName = hmcPartition.getIp() == null ? hmcPartition.getName() : hmcPartition.getIp();
                alarm.setTargetName(targetName);
            }
        }
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Object object) {
        if (MonitorResourceType.MONITOR_HMC_SERVER.name().equals(ruleGroup.getResourceType())) {
            HmcServer hmcServer = (HmcServer) object;
            return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(hmcServer.getId()), ruleGroup.getTenantId());
        }
        if (MonitorResourceType.MONITOR_HMC_VIOS.name().equals(ruleGroup.getResourceType()) ||
                MonitorResourceType.MONITOR_HMC_AIX.name().equals(ruleGroup.getResourceType())) {
            HmcPartition hmcPartition = (HmcPartition) object;
            return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(hmcPartition.getId()), ruleGroup.getTenantId());
        }
        return true;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_HMC_SERVER, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_HMC_AIX, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_HMC_VIOS, this);
    }
}

