package com.bocloud.cms.service;

import com.bocloud.cms.entity.MonitorAllocationRatio;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.QueryService;
import com.bocloud.cms.model.LineChartResult;
import com.bocloud.cms.model.LineChartResult.LineChartParam;
import com.bocloud.cms.model.LineChartResult.Values;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.AllocationRatioRepository;
import com.bocloud.cms.service.internal.CmpCloudVendorService;
import com.bocloud.ims.entity.CloudVendor;
import com.megatron.common.model.GeneralResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * prometheus监控图表
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
@Slf4j
public class QueryServiceImpl implements QueryService {


    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private AllocationRatioRepository allocationRatioRepository;
    @Autowired
    private CmpCloudVendorService cmpCloudVendorService;

    /**
     * 监控折线图：时间-值
     * @param lineChartParam
     * @return
     */
    private LineChartResult lineChart(LineChartParam lineChartParam) {
        List<PrometheusResult> list = prometheusService.queryRange(lineChartParam.getExpr(), lineChartParam.getStart(),
                lineChartParam.getEnd(), lineChartParam.getStep());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<String> keys = new ArrayList<>();
        List<Values> valuesList = new ArrayList<>();
        list = Optional.ofNullable(list).orElse(new ArrayList<>());
        list.forEach(prometheusResult -> {
            String name = lineChartParam.getName();
            if (lineChartParam.getName().indexOf("{") != -1) {
                String paramName = lineChartParam.getName().substring(lineChartParam.getName().indexOf("{") + 1, lineChartParam.getName().indexOf("}"));
                Object labelName = prometheusResult.getMetric().get(paramName.trim());
                if (labelName != null) {
                    name = lineChartParam.getName().replace("{" + paramName + "}", String.valueOf(labelName));
                }
            } else if (StringUtils.startsWith(lineChartParam.getName(), "$")) {
                Object labelName = prometheusResult.getMetric().get(lineChartParam.getName().substring(1));
                if (labelName != null) {
                    name = String.valueOf(labelName);
                }
            }
            List<Object> data = new ArrayList<>();
            for (Object[] value : prometheusResult.getValues()) {
                if (valuesList.isEmpty()) {
                    keys.add(sdf.format(new Date(Integer.parseInt(String.valueOf(value[0])) * 1000L)));
                }
                data.add(String.valueOf(value[1]));
            }
            valuesList.add(new Values(name, data));
        });
        return new LineChartResult(keys, valuesList.toArray(new Values[valuesList.size()]), null);
    }

    @Override
    public LineChartResult chart(List<String> exprs, List<String> names, String start, String end, String step, String unit) {
        LineChartResult result = null;
        for (int j = 0; j < exprs.size(); j++) {
            LineChartParam param = new LineChartParam();
            param.setName(names.get(j));
            param.setExpr(exprs.get(j));
            param.setStart(start);
            param.setEnd(end);
            param.setStep(step);
            LineChartResult lineChartResult = lineChart(param);
            if (result == null) {
                result = lineChartResult;
                result.setUnit(unit);
                continue;
            }
            Values[] values = result.getValues();
            Values[] newValues = new Values[lineChartResult.getValues().length + values.length];
            for (int i = 0; i < values.length; i++) {
                newValues[i] = values[i];
            }
            for (int i = 0; i < lineChartResult.getValues().length; i++) {
                newValues[values.length + i] = lineChartResult.getValues()[i];
            }
            result.setValues(newValues);
        }
        return result;
    }


    @Override
    public GeneralResult vendor(Long id) {
        try {
            Map<String, Object> result = new HashMap<>(16, 0.75F);
            MonitorAllocationRatio ratio = allocationRatioRepository.query(id);
            long cpuRatio = ratio != null ? ratio.getVcpuRatio() : 1L;
            long ramRatio = ratio != null ? ratio.getRamRatio() : 1L;
            long diskRatio = ratio != null ? ratio.getDiskRatio() : 1L;

            Double freeVcpu = prometheusService
                    .simpleQuery("sum(bocloud_openstack_hypervisor_vcpus_total{vendor_id='" + id + "'})*" + cpuRatio + "- sum(bocloud_openstack_hypervisor_vcpus_used{vendor_id='" + id + "'})");
            Double freeMem = prometheusService
                    .simpleQuery("round((sum(bocloud_openstack_hypervisor_memory_mbs_total{vendor_id='" + id + "'}) *" + ramRatio + " - sum(bocloud_openstack_hypervisor_memory_mbs_used{vendor_id='" + id + "'}))/1024,0.01)");
            Double freeDisk = prometheusService
                    .simpleQuery("round(sum(bocloud_openstack_hypervisor_disk_gbs_total{vendor_id='" + id + "'}) *" + diskRatio + "- sum(bocloud_openstack_hypervisor_disk_gbs_used{vendor_id='" + id + "'}),0.01)");
            Double vcpuUsed = prometheusService
                    .simpleQuery("sum(bocloud_openstack_hypervisor_vcpus_used{vendor_id='" + id + "'})");
            Double memUsed = prometheusService
                    .simpleQuery("round(sum(bocloud_openstack_hypervisor_memory_mbs_used{vendor_id='" + id + "'})/1024,0.01)");
            Object diskUsed = prometheusService
                    .simpleQuery("round(sum(bocloud_openstack_hypervisor_disk_gbs_used{vendor_id='" + id + "'}),0.01)");

            List<Object> vcpu = new ArrayList<>();
            Map<String, Object> map1 = new HashMap<>(16, 0.75F);
            map1.put("unit", "C");
            map1.put("name", "空闲");
            map1.put("value", null == freeVcpu ? "" : freeVcpu);
            vcpu.add(map1);
            Map<String, Object> map2 = new HashMap<>(16, 0.75F);
            map2.put("unit", "C");
            map2.put("name", "已使用");
            map2.put("value", null == vcpuUsed ? "" : vcpuUsed);
            vcpu.add(map2);

            List<Object> ram = new ArrayList<>();
            Map<String, Object> map3 = new HashMap<>(16, 0.75F);
            map3.put("unit", "GB");
            map3.put("name", "空闲");
            map3.put("value", null == freeMem ? "" : freeMem);
            ram.add(map3);
            Map<String, Object> map4 = new HashMap<>(16, 0.75F);
            map4.put("unit", "GB");
            map4.put("name", "已使用");
            map4.put("value", null == memUsed ? "" : memUsed);
            ram.add(map4);

            List<Object> disk = new ArrayList<>();
            Map<String, Object> map5 = new HashMap<>(16, 0.75F);
            map5.put("unit", "GB");
            map5.put("name", "空闲");
            map5.put("value", null == freeDisk ? "" : freeDisk);
            disk.add(map5);
            Map<String, Object> map6 = new HashMap<>(16, 0.75F);
            map6.put("unit", "GB");
            map6.put("name", "已使用");
            map6.put("value", null == diskUsed ? "" : diskUsed);
            disk.add(map6);

            result.put("vcpu", vcpu);
            result.put("ram", ram);
            result.put("disk", disk);
            result.put("vpcuAlloctionRate", cpuRatio);
            result.put("memAlloctionRate", ramRatio);
            result.put("diskAlloctionRate", diskRatio);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("Query OpenStack detail information fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult download(Long id, String uuid) {
        Map<String, Object> result = new HashMap<>(16, 0.75F);

        GeneralResult<CloudVendor> detail = cmpCloudVendorService.detail(id);
        MonitorAllocationRatio ratio = allocationRatioRepository.query(id);
        long cpuRatio = ratio != null ? ratio.getVcpuRatio() : 1L;
        long ramRatio = ratio != null ? ratio.getRamRatio() : 1L;
        long diskRatio = ratio != null ? ratio.getDiskRatio() : 1L;

        Double totalVcpu = prometheusService
                .simpleQuery("sum(bocloud_openstack_hypervisor_vcpus_total{vendor_id='" + uuid + "'})*" + cpuRatio);
        Double totalMem = prometheusService
                .simpleQuery("round((sum(bocloud_openstack_hypervisor_memory_mbs_total{vendor_id='" + uuid + "'}) *" + ramRatio + ")/1024,0.01)");
        Double totalDisk = prometheusService
                .simpleQuery("round(sum(bocloud_openstack_hypervisor_disk_gbs_total{vendor_id='" + uuid + "'}) *" + diskRatio + ",0.01)");

        Double vcpuUsed = prometheusService
                .simpleQuery("sum(bocloud_openstack_hypervisor_vcpus_used{vendor_id='" + uuid + "'})");
        Double memUsed = prometheusService
                .simpleQuery("round(sum(bocloud_openstack_hypervisor_memory_mbs_used{vendor_id='" + uuid + "'})/1024,0.01)");
        Double diskUsed = prometheusService
                .simpleQuery("round(sum(bocloud_openstack_hypervisor_disk_gbs_used{vendor_id='" + uuid + "'}),0.01)");

        List<Object> vcpu = new ArrayList<>();
        Map<String, Object> map2 = new HashMap<>(16, 0.75F);
        map2.put("unit", "C");
        map2.put("name", "已使用");
        map2.put("value", null == vcpuUsed ? "" : vcpuUsed);
        vcpu.add(map2);
        Map<String, Object> map3 = new HashMap<>(16, 0.75F);
        map3.put("unit", "C");
        map3.put("name", "总数");
        map3.put("value", null == totalVcpu ? "" : totalVcpu);
        vcpu.add(map3);
        List<Object> ram = new ArrayList<>();
        Map<String, Object> map5 = new HashMap<>(16, 0.75F);
        map5.put("unit", "GB");
        map5.put("name", "已使用");
        map5.put("value", null == memUsed ? "" : memUsed);
        ram.add(map5);
        Map<String, Object> map6 = new HashMap<>(16, 0.75F);
        map6.put("unit", "GB");
        map6.put("name", "总数");
        map6.put("value", null == totalMem ? "" : totalMem);
        ram.add(map6);
        List<Object> disk = new ArrayList<>();
        Map<String, Object> map8 = new HashMap<>(16, 0.75F);
        map8.put("unit", "GB");
        map8.put("name", "已使用");
        map8.put("value", null == diskUsed ? "" : diskUsed);
        disk.add(map8);
        Map<String, Object> map9 = new HashMap<>(16, 0.75F);
        map9.put("unit", "GB");
        map9.put("name", "总数");
        map9.put("value", null == totalDisk ? "" : totalDisk);
        disk.add(map9);

        result.put("name", detail.getData().getName());
        result.put("vcpu", vcpu);
        result.put("ram", ram);
        result.put("disk", disk);
        return new GeneralResult<>(true, result, "查询成功");

    }

}
