package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceStatisticsService;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.bocloud.ims.entity.CloudVendor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class ResourceStatisticsServiceImpl implements ResourceStatisticsService {

    private static final Logger logger = LoggerFactory.getLogger(ResourceStatisticsServiceImpl.class);

    @Autowired
    private CmpCloudServerService vendorService;

    @Autowired
    private PrometheusService prometheusService;

    @Override
    public GeneralResult usage(String type) {
        // 定义返回对象
        Map<String, Object> resultMap = new HashMap<>();
        List<String> keys = new ArrayList<>();
        resultMap.put("keys",keys);
        List<JSONObject> values = new ArrayList<>();
        resultMap.put("values",values);
        // 查询所有站点
        List<Param> paramList = new ArrayList<>();
        paramList.add(new Param(MapTools.simpleMap("is_alarm", 1), Sign.EQ));
        GeneralResult vendorResult = vendorService.list(new Pager(1, Integer.MAX_VALUE, paramList,
                null));
        if (vendorResult.isFailed()) {
            return new GeneralResult<>(false, "查询站点失败");
        }
        // 查询到平台，取出平台id列表，然後使用平台id再去查平台下的Server
        JSONObject data = (JSONObject) vendorResult.getData();
        List<CloudVendor> cloudVendors = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        JSONObject cpuObj = new JSONObject();
        cpuObj.put("name", "CPU%");
        JSONArray cpuArr = new JSONArray();
        JSONObject memObj = new JSONObject();
        memObj.put("name", "内存%");
        JSONArray memArr = new JSONArray();
        JSONObject diskObj = new JSONObject();
        diskObj.put("name", "存储%");
        JSONArray diskArr = new JSONArray();
        // 通过Prometheus查询站点的资源用量
        for (CloudVendor cloudVendor : cloudVendors) {
            keys.add(cloudVendor.getName());
            Double cpu = prometheusService.simpleQuery("fusioncompute_resource_percent_cpu");
            Double mem = prometheusService.simpleQuery("fusioncompute_resource_percent_mem");
            Double disk = prometheusService.simpleQuery("fusioncompute_resource_percent_disk");
            if (null == cpu || null == mem || null == disk) {
                return new GeneralResult<>(false, "查询指标失败");
            }

            cpuArr.add(cpu);
            memArr.add(mem);
            diskArr.add(disk);
        }
        cpuObj.put("data",cpuArr);
        memObj.put("data",memArr);
        diskObj.put("data",diskArr);

        switch (type) {
            case "cpu":
                values.add(cpuObj);
                break;
            case "mem":
                values.add(memObj);
                break;
            case "disk":
                values.add(diskObj);
                break;
        }

        return new GeneralResult<>(true, resultMap, "查询成功");
    }


    private static List<String> getLatestWeek() {
        List<String> weeks = new ArrayList<>();
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        for (int i = 6; i >= 0; i--) {
            calendar.setTime(new Date());
            //i ---->后几天，-i ---->前几天
            calendar.add(Calendar.DATE, -i);
            //calendar.add(Calendar.DATE,-i);
            Date time = calendar.getTime();
            String toTime = formatter.format(time);
            weeks.add(toTime);
        }
        return weeks;
    }

    public static void main(String[] args) throws ParseException {
        List<String> latestWeek = getLatestWeek();
        for (String s : latestWeek) {
            Date parse = new SimpleDateFormat("yyyy-MM-dd").parse(s);
            System.out.println(parse.getTime());
        }
    }
}
