package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.SanFabricService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.SanFabricPortModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.ciscoswitch.SanFabricPortService;
import com.bocloud.nms.entity.SanFabricPort;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * SanFabricPort实体类
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Service
public class SanFabricServiceImpl implements SanFabricService, InitializingBean {

    @Autowired
    private SanFabricPortService sanFabricPortService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public GeneralResult listOfPorts(Pager pager, RequestContext context, HttpServletRequest request) {
        GeneralResult result = sanFabricPortService.listOfPorts(pager, request.getHeader(Common.TOKEN));
        JSONObject data = (JSONObject) result.getData();
        List<SanFabricPortModel> list = data.getJSONArray(Common.ROWS).toJavaList(SanFabricPortModel.class);
        if (list.isEmpty()) {
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_CISCO_SANFABRIC.name(), targets, context);
        for (SanFabricPortModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<SanFabricPort> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);

        List<Param> params = Collections.singletonList(param);
        GeneralResult result = sanFabricPortService.listOfPorts(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(SanFabricPort.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(SanFabricPort sanFabricPort, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("vendorId", String.valueOf(sanFabricPort.getVendorId()));
        params.put("ifName", String.valueOf(sanFabricPort.getWwn()));
        return params;
    }

    @Override
    public SanFabricPort parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
        String ifName = String.valueOf(alert.getLabels().get("ifName"));
        List<Param> params = new ArrayList<>(1);
        params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("wwn", ifName), Sign.EQ));
        GeneralResult result = sanFabricPortService.listOfPorts(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<SanFabricPort> list = data.getJSONArray(Common.ROWS).toJavaList(SanFabricPort.class);
        Assert.isTrue(list != null && !list.isEmpty(), "SanFabricPort not found, wwn:" + ifName + ", vendorId:" + vendorId);
        return list.get(0);
    }

    @Override
    public void destroy(SanFabricPort object, String catalog) throws Exception {

    }

    @Override
    public void wrapAlarm(Alarm alarm, SanFabricPort object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        alarm.setTargetName(object.getWwn());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, SanFabricPort object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_CISCO_SANFABRIC, this);
    }

}
