package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.ServiceService;
import com.bocloud.cms.model.OpenStackService;
import com.bocloud.cms.model.OpenStackServiceModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.PrometheusResult;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 服务Service实现类
 *
 * @author zhangdi
 * @since Feb 28,2020
 */
@Service
public class ServiceServiceImpl implements ServiceService, InitializingBean {

    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;


    @Override
    public GeneralResult list(String name, String status, Long vendorId, RequestContext context) {
        if (status==null) { status=""; }
        if (name==null) { name=""; }
        if (status.length() > 0) {
            status = ("UP").equals(status) ? "==1" : "==0";
        }
        List<OpenStackServiceModel> result = new ArrayList<>();
        List<String> targets = new ArrayList<>();
        List<PrometheusResult> novas;
        if (vendorId!=null){
            novas = prometheusService.query("bocloud_openstack_nova_agent_state{vendor_id = '" + vendorId + "',service=~'.*" + name + ".*'}" + status);
        }else{
            novas = prometheusService.query("bocloud_openstack_nova_agent_state{service=~'.*" + name + ".*'}" + status);
        }
        for (PrometheusResult nova : novas) {
            OpenStackServiceModel openStackService = new OpenStackServiceModel();
            Map<String, Object> tag = new HashMap<>(16, 0.75F);
            openStackService.setName(String.valueOf(nova.getMetric().get("service")));
            tag.put("status", nova.getMetric().get("status"));
            tag.put("host", nova.getMetric().get("host"));
            openStackService.setTag(tag);
            openStackService.setId(String.valueOf(nova.getMetric().get("id")));
            openStackService.setStatus("1".equals(nova.getValue()[1]) ? "UP" : "DOWN");
            targets.add(String.valueOf(nova.getMetric().get("id")));
            result.add(openStackService);
        }

        List<PrometheusResult> neutrons;
        if (vendorId!=null){
            neutrons = prometheusService.query("bocloud_openstack_neutron_agent_state{vendor_id = '" + vendorId + "',service=~'.*" + name + ".*'}" + status);
        }else{
            neutrons = prometheusService.query("bocloud_openstack_neutron_agent_state{service=~'.*" + name + ".*'}" + status);
        }
        for (PrometheusResult neutron : neutrons) {
            OpenStackServiceModel openStackService = new OpenStackServiceModel();
            Map<String, Object> tag = new HashMap<>(16, 0.75F);
            openStackService.setName(String.valueOf(neutron.getMetric().get("service")));
            tag.put("status", neutron.getMetric().get("adminStateUp"));
            tag.put("host", neutron.getMetric().get("host"));
            openStackService.setTag(tag);
            openStackService.setId(String.valueOf(neutron.getMetric().get("id")));
            openStackService.setStatus("1".equals(neutron.getValue()[1]) ? "UP" : "DOWN");
            targets.add(String.valueOf(neutron.getMetric().get("id")));
            result.add(openStackService);
        }

        List<PrometheusResult> cinders;
        if (vendorId!=null){
            cinders = prometheusService.query("bocloud_openstack_cinder_service_state{vendor_id = '" + vendorId + "',service=~'.*" + name + ".*'}" + status);
        }else{
            cinders = prometheusService.query("bocloud_openstack_cinder_service_state{service=~'.*" + name + ".*'}" + status);
        }
        for (PrometheusResult cinder : cinders) {
            OpenStackServiceModel openStackService = new OpenStackServiceModel();
            Map<String, Object> tag = new HashMap<>(16, 0.75F);
            openStackService.setName(String.valueOf(cinder.getMetric().get("service")));
            tag.put("status", cinder.getMetric().get("status"));
            tag.put("host", cinder.getMetric().get("host"));
            openStackService.setTag(tag);
            openStackService.setId(String.valueOf(cinder.getMetric().get("id")));
            openStackService.setStatus("1".equals(cinder.getValue()[1]) ? "UP" : "DOWN");
            targets.add(String.valueOf(cinder.getMetric().get("id")));
            result.add(openStackService);
        }
        for (OpenStackServiceModel openStackServiceModel : result) {
            openStackServiceModel.setAlarmEnable(true);
        }
        return new GeneralResult<>(true, result, "查询成功");
    }

    /**
     * 将所有的服务转换成为OpenStackServiceModel对象
     *  查询出已绑定的资源
     * @return
     */
    @Override
    public List<OpenStackServiceModel> listByGroupId() {
        GeneralResult list = this.list(null, null, null, new RequestContext());
        JSONObject parse = (JSONObject) JSONObject.parse(list.toString());
        List<OpenStackServiceModel> services = parse.getJSONArray("data").toJavaList(OpenStackServiceModel.class);
        for (OpenStackServiceModel service : services) {
            JSONObject jsonObject = JSONObject.parseObject(service.getTag().toString());
            service.setTarget(jsonObject.get("host").toString());
        }
        return services;
    }

    @Override
    public List<OpenStackService> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<OpenStackServiceModel> openStackServiceModels = listByGroupId();
        List<OpenStackService> openStackServices = new ArrayList<>();
        Optional.ofNullable(openStackServiceModels).orElse(new ArrayList<>()).forEach(openStackServiceModel -> {
            Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> {
                openStackServices.add(openStackServiceModel);
            });
        });
        return openStackServices;
    }

    @Override
    public Map<String, String> getPrometheusExprParam(OpenStackService targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("service",targetObject.getName());
        params.put("host", targetObject.getTarget());
        return params;
    }

    @Override
    public OpenStackService parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        OpenStackService openStackService = new OpenStackService();
        openStackService.setId(String.valueOf(alert.getLabels().get("id")));
        return openStackService;
    }

    @Override
    public void destroy(OpenStackService object, String catalog) throws Exception {

    }

    @Override
    public void wrapAlarm(Alarm alarm, OpenStackService object) {
        // 设置个性化告警
        alarm.setTargetName(object.getId());
        alarm.setTarget(object.getId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, OpenStackService object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_OPENSTACK_SERVICE, this);
    }
}
