package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.MonitoringAble;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.service.internal.CmpCloudVendorService;
import com.bocloud.cms.service.internal.CmpServerService;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;

/**
 * @author wpj
 * @since 2022/7/20
 */


@Slf4j
@Service
public class SmartxHostServiceImpl implements MonitoringAble<Server>, InitializingBean {

    @Autowired
    private CmpServerService cmpServerService;

    @Autowired
    private CmpCloudVendorService cmpCloudVendorService;

    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private AlarmService alarmService;

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_SMARTX_HOST, this);
    }

    @Override
    public List<Server> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = cmpServerService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(Server.class);
    }


    @Override
    public Map<String, String> getPrometheusExprParam(Server server, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("host_id", server.getLocalId() + "");
        return params;
    }

    @Override
    public Server parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String hostId = String.valueOf(alert.getLabels().get("host_id"));
        String vendorUuId = String.valueOf(alert.getLabels().get("vendorId"));
        //查询平台获取vendorId
        List<Param> paramsVendor = new ArrayList<>(2);
        paramsVendor.add(new Param(MapTools.simpleMap("id", Long.parseLong(vendorUuId)), Sign.EQ));
        GeneralResult vendor = cmpCloudVendorService.list(new Pager(1, 1, paramsVendor, null));
        Assert.isTrue(vendor.isSuccess(), vendor.getMessage());
        JSONObject vendorData = (JSONObject) vendor.getData();
        List<CloudVendor> vendors = vendorData.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        //获取宿主机
        List<Param> params = new ArrayList<>(2);
        params.add(new Param(MapTools.simpleMap("localId", hostId), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("vendorId", vendors.get(0).getId()), Sign.EQ));
        GeneralResult result = cmpServerService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        Assert.isTrue(list != null && !list.isEmpty(), "smartx server not found,id:" + hostId + ",vendorId:" + vendors.get(0).getId());
        return list.get(0);
    }

    @Override
    public void destroy(Server server, String catalog) throws Exception {
        log.info("destroy smartx host " + server.getId());
        alarmService.solveByCloudServer(server.getId(),catalog,1L,"系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, Server server) {
        alarm.setTarget(server.getId());
        alarm.setTargetName(server.getName());
        alarm.setVendorId(server.getVendorId());
        alarm.setOwnerId(server.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Server object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }
}

