package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.TceVolumeService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.VolumeModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author  hya
 * @date 2022/5/10 11:11
 */
@Service
@Slf4j
public class TceVolumeServiceImpl implements TceVolumeService, InitializingBean {

    @Autowired
    private CmpCloudServerService cmpCloudServerService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private AlarmService alarmService;

    @Override
    public GeneralResult listOfVolume(Pager pager, RequestContext context, HttpServletRequest request) {
        GeneralResult result = cmpCloudServerService.listOfVolume(pager, request.getHeader(Common.TOKEN));
        JSONObject data = (JSONObject) result.getData();
        List<VolumeModel> list = data.getJSONArray(Common.ROWS).toJavaList(VolumeModel.class);
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_TCE_CBS.name(), targets, context);
        for (VolumeModel volumeModel : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(volumeModel.getId().toString()))
                    .findAny().isPresent();
            volumeModel.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<VolumeModel> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetIds).substring(1, JSONObject.toJSONString(targetIds).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = cmpCloudServerService.listOfVolume(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(VolumeModel.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(VolumeModel targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>();
        params.put("disk_id", targetObject.getVolumeUuid());
        params.put("vendorId", targetObject.getVendorId().toString());
        return params;
    }

    @Override
    public VolumeModel parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String diskId = String.valueOf(alert.getLabels().get("disk_id"));
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("volumeUuid", diskId), Sign.EQ));
        GeneralResult result = cmpCloudServerService.listOfVolume(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<VolumeModel> list = data.getJSONArray(Common.ROWS).toJavaList(VolumeModel.class);
        // Assert.isTrue(list != null && !list.isEmpty(), "tce disk not found,diskId:" + diskId);
        if (list.size()==0){
            log.info("TceDisk---->diskId----->" + diskId);
            return null;
        }
        return list.get(0);
    }

    @Override
    public void destroy(VolumeModel object, String catalog) throws Exception {
        log.info("destroy tce disk : " + object.getId());
        alarmService.solveByCloudServer(object.getId(),catalog,1L,"系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, VolumeModel object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getName() + "(" + object.getVolumeUuid() + ")";
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, VolumeModel object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_TCE_CBS, this);
    }
}
