package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.TopService;
import com.bocloud.cms.model.LineChartResult;
import com.bocloud.cms.model.LineChartResult.TopParams;
import com.bocloud.cms.model.LineChartResult.Values;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.model.TopBaseEntity;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.bocloud.cms.service.internal.CmpCloudVendorService;
import com.bocloud.cms.service.internal.CmpServerService;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;
import com.bocloud.ims.enums.CloudProvider;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.*;

/**
 * Top5监控图表
 *
 * @author zhangyf
 * @version 5.6.0
 * @since 2022/2/21 18:01
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class TopServiceImpl implements TopService {

    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("top-pool-%d").build();
    private final ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1,
            0L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<>(), nameThreadFactory);

    private final CmpCloudServerService cmpCloudServerService;
    private final CmpServerService cmpServerService;
    private final CmpCloudVendorService cmpCloudVendorService;
    private final PrometheusService prometheusService;

    /**
     * 主机CpuTop
     *
     * @param vendorId
     * @param limit
     * @return
     */
    @Override
    public LineChartResult topOfHostCpu(Long vendorId, Integer limit) {
        if (null == vendorId) {
            return hostLineChart("宿主机CPU利用率", "round(sort(topk(" + limit + ",vsphere_host_cpu_usage_average{cpu='instance-total'} or (1 - avg(irate(node_cpu_seconds_total{mode='idle',vendor_id!=''}[5m])) by (host_id))*100 or bocloud_fusioncloud_host_cpuUsage)),0.01)");
        }
        CloudVendor vendor = cmpCloudVendorService.detail(vendorId).getData();
        switch (CloudProvider.valueOf(vendor.getType())) {
            case OPENSTACK:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",(1 - avg(irate(node_cpu_seconds_total{mode='idle',vendor_id='" + vendor.getId() + "'}[5m])) by (host_id))*100)),0.01)");
            case VMWARE:
                String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",vsphere_host_cpu_usage_average{cpu='instance-total',vcenter='" + address + "'})),0.01)");
            case MANAGEONE:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",bocloud_fusioncloud_host_cpuUsage{vendorId='" + vendorId + "'})),0.01)");
            case EASYSTACK:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",easystack_host_node_cpu_utilization_total{EsCloud='" + vendor.getUuid() + "'})),0.01)");
            case CNWARE:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",cnware_host_cpu_rate{vendorId='" + vendorId + "'})),0.01)");
            case ZSTACK:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",zstack_host_used_cpu_rate{vendorId='" + vendorId + "'})),0.01)");
            case SMARTX:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",smartx_host_cpu_overall_usage_percent{vendorId='" + vendorId + "'})),0.01)");
            case KVM:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",node_memory_usage{vendor_id='" + vendorId + "'})),0.01)");
            default:
                return new LineChartResult(null, new Values[]{new Values("宿主机cpu利用率", null)}, null);
        }
    }

    /**
     * 主机MemTop
     *
     * @param vendorId
     * @param limit
     * @return
     */
    @Override
    public LineChartResult topOfHostMem(Long vendorId, Integer limit) {
        if (null == vendorId) {
            return hostLineChart("宿主机内存利用率", "round(sort(sort_desc(" + limit + ",vsphere_host_mem_usage_average or 100 - ((node_memory_MemFree_bytes+node_memory_Cached_bytes+node_memory_Buffers_bytes{vendor_id!=''})/node_memory_MemTotal_bytes) * 100 or bocloud_fusioncloud_host_memoryUsage)),0.01)");
        }
        CloudVendor vendor = cmpCloudVendorService.detail(vendorId).getData();
        switch (CloudProvider.valueOf(vendor.getType())) {
            case OPENSTACK:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ", 100 - ((node_memory_MemFree_bytes+node_memory_Cached_bytes+node_memory_Buffers_bytes{vendor_id='" + vendorId + "'})/node_memory_MemTotal_bytes) * 100)),0.01)");
            case VMWARE:
                String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",vsphere_host_mem_usage_average{vcenter='" + address + "'})),0.01)");
            case MANAGEONE:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",bocloud_fusioncloud_host_memoryUsage{vendorId='" + vendorId + "'})),0.01)");
            case EASYSTACK:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit +
                        ",(easystack_host_node_memory_usage{EsCloud='" + vendor.getUuid() + "'} / easystack_host_node_memory_total{EsCloud='" + vendor.getUuid() + "'}) * 100)),0.01)");
            case CNWARE:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",cnware_host_mem_rate{vendorId='" + vendorId + "'})),0.01)");
            case ZSTACK:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",zstack_host_used_mem_rate{vendorId='" + vendorId + "'})),0.01)");
            case SMARTX:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",smartx_host_memory_usage_percent{vendorId='" + vendorId + "'})),0.01)");
            case KVM:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",node_cpu_usage{vendor_id='" + vendorId + "'})),0.01)");
            default:
                return new LineChartResult(null, new Values[]{new Values("宿主机内存利用率", null)}, null);
        }
    }

    /**
     * 虚拟机CpuTop
     *
     * @param context  当前用户上下文
     * @param vendorId
     * @param limit
     * @param request
     * @return
     */
    @Override
    public LineChartResult topOfvmCpu(RequestContext context, Long vendorId, Integer limit, HttpServletRequest request, String vendorType) {
        String expr;
        if (null == vendorId && null == vendorType) {
            vendorType = String.valueOf(CloudProvider.MANAGEONE);
        }
        if (null != vendorType) {
            switch (CloudProvider.valueOf(vendorType)) {
                case OPENSTACK:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",avg(irate(libvirt_domain_info_cpu_time_seconds_total{}[5m])) by (vendor_id,instanceId,instanceName)/ avg(libvirt_domain_info_virtual_cpus) by (vendor_id,instanceId,instanceName))*100),0.01)";
                    break;
                case VMWARE:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",vsphere_vm_cpu_usage_average{cpu='instance-total'})),0.01)";
                    break;
                case MANAGEONE:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",bocloud_fusioncloud_vm_cpuUsage{})),0.01)";
                    break;
                case EASYSTACK:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",easystack_vm_cpu_util{})),0.01)";
                    break;
                case HWACCESS:
                    expr = "round(sort_desc(topk(" + limit * 5 + ",fusioncompute_vm_cpu_usage{})),0.01)";
                    log.info(expr);
                    break;
                case APSARASTACK:
                    expr = "round(sort_desc(topk(" + limit + ",apsarastack_ecs_CPUUtilization_average{})),0.01)";
                    log.info(expr);
                    break;
                case ALIYUN:
                    expr = "round(sort_desc(topk(" + limit + ",aliyun_ecs_CPUUtilization_average{})),0.01)";
                    log.info(expr);
                    break;
                case HCSO:
                    expr = "round(sort_desc(topk(" + limit + ",hcso_vm_sys_ecs_cpu_util{})),0.01)";
                    log.info(expr);
                    break;
                case CAS:
                    expr = "round(sort_desc(topk(" + limit + ",h3c_vm_cpu_rate{})),0.01)";
                    log.info(expr);
                    break;
                case KVM:
                    expr = "round(sort_desc(topk(" + limit + ",kvm_cpu_usage{})),0.01)";
                    log.info(expr);
                    break;
                case CNWARE:
                    expr = "round(sort_desc(topk(" + limit + ",cnware_vm_cpu_rate{})),0.01)";
                    log.info(expr);
                    break;
                case ZSTACK:
                    expr = "round(sort_desc(topk(" + limit + ",zstack_vm_CPUAverageUsedUtilization{})),0.01)";
                    log.info(expr);
                    break;
                case SMARTX:
                    expr = "round(sort_desc(topk(" + limit + ",smartx_elf_vm_cpu_overall_usage_percent{})),0.01)";
                    log.info(expr);
                    break;
                case QCLOUD:
                    expr = "round(sort_desc(topk(" + limit + ",qcloud_cvm_cpu{})),0.01)";
                    log.info(expr);
                    break;
                case CLOUDOS:
                    expr = "round(sort_desc(topk(" + limit + ",cloudOs_vm_cpu_util{})),0.01)";
                    log.info(expr);
                    break;
                case CTSTACK:
                    expr = "round(sort_desc(topk(" + limit + ",tianyi_vm_cpu_util{})),0.01)";
                    log.info(expr);
                    break;
                default:
                    expr = "";
            }
        } else {
            CloudVendor vendor = cmpCloudVendorService.detail(vendorId).getData();
            switch (CloudProvider.valueOf(vendor.getType())) {
                case OPENSTACK:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",avg(irate(libvirt_domain_info_cpu_time_seconds_total{vendor_id='" + vendor.getId() + "'}[5m])) by (vendor_id,instanceId,instanceName)/ avg(libvirt_domain_info_virtual_cpus) by (vendor_id,instanceId,instanceName))*100),0.01)";
                    break;
                case VMWARE:
                    String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                    expr = "round(sort_desc(topk(" + limit * 3 + ",vsphere_vm_cpu_usage_average{cpu='instance-total',vcenter='" + address + "'})),0.01)";
                    break;
                case MANAGEONE:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",bocloud_fusioncloud_vm_cpuUsage{vendorId='" + vendor.getId() + "'})),0.01)";
                    break;
                case EASYSTACK:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",easystack_vm_cpu_util{EsCloud='" + vendor.getUuid() + "'})),0.01)";
                    break;
                case HWACCESS:
                    expr = "round(sort_desc(topk(" + limit * 5 + ",fusioncompute_vm_cpu_usage{vendor_id = '" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case APSARASTACK:
                    expr = "round(sort_desc(topk(" + limit + ",apsarastack_ecs_CPUUtilization_average{vendorId = '" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case ALIYUN:
                    expr = "round(sort_desc(topk(" + limit + ",aliyun_ecs_CPUUtilization_average{vendorId = '" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case HCSO:
                    expr = "round(sort_desc(topk(" + limit + ",hcso_vm_sys_ecs_cpu_util{vendor_id = '" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case CAS:
                    expr = "round(sort_desc(topk(" + limit + ",h3c_vm_cpu_rate{vendor_id = '" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case KVM:
                    expr = "round(sort_desc(topk(" + limit + ",kvm_cpu_usage{vmCloud = '" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case CNWARE:
                    expr = "round(sort_desc(topk(" + limit + ",cnware_vm_cpu_rate{vendorId ='" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case ZSTACK:
                    expr = "round(sort_desc(topk(" + limit + ",zstack_vm_CPUAverageUsedUtilization{vendorId ='" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case SMARTX:
                    expr = "round(sort_desc(topk(" + limit + ",smartx_elf_vm_cpu_overall_usage_percent{vendorId ='" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case CLOUDOS:
                    expr = "round(sort_desc(topk(" + limit + ",cloudOs_vm_cpu_util{vendorId ='" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case CTSTACK:
                    expr = "round(sort_desc(topk(" + limit + ",tianyi_vm_cpu_util{vendor_id ='" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                default:
                    expr = "";
            }
        }
        return vmLineChart(request, "云主机CPU利用率", expr, limit);
    }

    /**
     * 虚拟机MemTop
     *
     * @param context  当前用户上下文
     * @param vendorId
     * @param limit
     * @param request
     * @return
     */
    @Override
    public LineChartResult topOfvmMem(RequestContext context, Long vendorId, Integer limit, HttpServletRequest request, String vendorType) {
        String expr;
        if (null == vendorId && null == vendorType) {
            vendorType = String.valueOf(CloudProvider.MANAGEONE);
        }
        if (null != vendorType) {
            switch (CloudProvider.valueOf(vendorType)) {
                case OPENSTACK:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",(1-libvirt_domain_memory_stat_unused/libvirt_domain_memory_stat_available{})*100)),0.01)";
                    break;
                case VMWARE:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",vsphere_vm_mem_usage_average{})),0.01)";
                    break;
                case MANAGEONE:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",bocloud_fusioncloud_vm_memoryUsage{})),0.01)";
                    break;
                case EASYSTACK:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",easystack_vm_memory_util{})),0.01)";
                    break;
                case HWACCESS:
                    expr = "round(sort_desc(topk(" + limit * 5 + ",fusioncompute_vm_mem_usage{})),0.01)";
                    log.info(expr);
                    break;
                case APSARASTACK:
                    expr = "round(sort_desc(topk(" + limit + ",apsarastack_ecs_memory_usedutilization_average{})),0.01)";
                    log.info(expr);
                    break;
                case ALIYUN:
                    expr = "round(sort_desc(topk(" + limit + ",aliyun_ecs_memory_usedutilization_average{})),0.01)";
                    log.info(expr);
                    break;
                case HCSO:
                    expr = "round(sort_desc(topk(" + limit + ",hcso_vm_agt_ecs_mem_usedPercent{})),0.01)";
                    log.info(expr);
                    break;
                case KVM:
                    expr = "round(sort_desc(topk(" + limit + ",kvm_memory_usage{})),0.01)";
                    log.info(expr);
                    break;
                case CNWARE:
                    expr = "round(sort_desc(topk(" + limit + ",cnware_vm_mem_rate{})),0.01)";
                    log.info(expr);
                    break;
                case ZSTACK:
                    expr = "round(sort_desc(topk(" + limit + ",zstack_vm_MemoryUsedInPercent{})),0.01)";
                    log.info(expr);
                    break;
                case SMARTX:
                    expr = "round(sort_desc(topk(" + limit + ",smartx_elf_vm_memory_usage_percent{})),0.01)";
                    log.info(expr);
                    break;
                case QCLOUD:
                    expr = "round(sort_desc(topk(" + limit + ",qcloud_cvm_memory{})),0.01)";
                    log.info(expr);
                    break;
                case CLOUDOS:
                    expr = "round(sort_desc(topk(" + limit + ",cloudOs_vm_memory_util{})),0.01)";
                    log.info(expr);
                    break;
                case CTSTACK:
                    expr = "round(sort_desc(topk(" + limit + ",tianyi_vm_mem_util{})),0.01)";
                    log.info(expr);
                    break;
                default:
                    expr = "";
            }
        } else {
            CloudVendor vendor = cmpCloudVendorService.detail(vendorId).getData();
            switch (CloudProvider.valueOf(vendor.getType())) {
                case OPENSTACK:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",(1-libvirt_domain_memory_stat_unused/libvirt_domain_memory_stat_available{vendor_id='" + vendor.getId() + "'})*100)),0.01)";
                    break;
                case VMWARE:
                    String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                    expr = "round(sort_desc(topk(" + limit * 3 + ",vsphere_vm_mem_usage_average{vcenter='" + address + "'})),0.01)";
                    break;
                case MANAGEONE:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",bocloud_fusioncloud_vm_memoryUsage{vendorId='" + vendor.getId() + "'})),0.01)";
                    break;
                case EASYSTACK:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",easystack_vm_memory_util{EsCloud='" + vendor.getUuid() + "'})),0.01)";
                    break;
                case HWACCESS:
                    expr = "round(sort_desc(topk(" + limit * 5 + ",fusioncompute_vm_mem_usage{vendor_id = '" + vendor.getId() + "'})),0.01)";
                    log.info(expr);
                    break;
                case APSARASTACK:
                    expr = "round(sort_desc(topk(" + limit + ",apsarastack_ecs_memory_usedutilization_average{vendorId = '" + vendor.getId() + "'})),0.01)";
                    log.info(expr);
                    break;
                case ALIYUN:
                    expr = "round(sort_desc(topk(" + limit + ",aliyun_ecs_memory_usedutilization_average{vendorId = '" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case HCSO:
                    expr = "round(sort_desc(topk(" + limit * 3 + ",hcso_vm_agt_ecs_mem_usedPercent{vendor_id = '" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case KVM:
                    expr = "round(sort_desc(topk(" + limit + ",kvm_memory_usage{vmCloud = '" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case CNWARE:
                    expr = "round(sort_desc(topk(" + limit + ",cnware_vm_mem_rate{vendorId ='" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case ZSTACK:
                    expr = "round(sort_desc(topk(" + limit + ",zstack_vm_MemoryUsedInPercent{vendorId ='" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case SMARTX:
                    expr = "round(sort_desc(topk(" + limit + ",smartx_elf_vm_memory_usage_percent{})),0.01)";
                    log.info(expr);
                    break;
                case CLOUDOS:
                    expr = "round(sort_desc(topk(" + limit + ",cloudOs_vm_memory_util{vendorId ='" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                case CTSTACK:
                    expr = "round(sort_desc(topk(" + limit + ",tianyi_vm_mem_util{vendor_id ='" + vendorId + "'})),0.01)";
                    log.info(expr);
                    break;
                default:
                    expr = "";
            }
        }
        return vmLineChart(request, "云主机内存利用率", expr, limit);
    }

    /**
     * 主机Top图表
     *
     * @param name
     * @param expr
     * @return
     */
    private LineChartResult hostLineChart(String name, String expr) {
        // 获取当前时间戳
        String time = Long.toString(System.currentTimeMillis()).substring(0, 10);
        List<PrometheusResult> list = prometheusService.query(expr, time);
        List<String> keys = new ArrayList<>(list.size());
        List<Object> data = new ArrayList<>(list.size());
        // ids列表，用于作监控详情信息展示
        Map<String, TopParams> params = new HashMap<>(16, 0.75F);
        List<Future<TopBaseEntity>> futures = new ArrayList<>();
        for (PrometheusResult result : list) {
            List<Param> paramList = new ArrayList<>();
            Future<TopBaseEntity> future = executorService.submit(() -> {
                CloudProvider vendorType = null;
                if (result.getMetric().containsKey("vendor_id")) {
                    GeneralResult<CloudVendor> vendorGeneralResult = cmpCloudVendorService.detail(Long.valueOf(String.valueOf(result.getMetric().get("vendor_id"))));
                    vendorType = CloudProvider.valueOf(vendorGeneralResult.getData().getType());
                }
                if (result.getMetric().containsKey("vendorId")) {
                    GeneralResult<CloudVendor> vendorGeneralResult = cmpCloudVendorService.detail(Long.valueOf(String.valueOf(result.getMetric().get("vendorId"))));
                    vendorType = CloudProvider.valueOf(vendorGeneralResult.getData().getType());
                }
                String hostName = "";
                if (CloudProvider.SMARTX == vendorType) {
                    //smartx
                    String hostId = String.valueOf(result.getMetric().get("host_id"));
                    paramList.add(new Param(MapTools.simpleMap("localId", hostId), Sign.EQ));
                } else if (result.getMetric().containsKey("vcenter")) {
                    String esxhostname = String.valueOf(result.getMetric().get("esxhostname"));
                    paramList.add(new Param(MapTools.simpleMap("name", esxhostname), Sign.EQ));
                } else if (result.getMetric().containsKey("host_id")) {
                    //openstack 使用id查询
                    String id = String.valueOf(result.getMetric().get("host_id"));
                    paramList.add(new Param(MapTools.simpleMap("id", id), Sign.EQ));
                } else if (result.getMetric().containsKey("uuid")) {
                    String uuid = String.valueOf(result.getMetric().get("uuid"));
                    paramList.add(new Param(MapTools.simpleMap("uuid", uuid), Sign.EQ));
                } else if (result.getMetric().containsKey("EsCloud")) {
                    // easystack 使用id查询
                    String uuid = String.valueOf(result.getMetric().get("id"));
                    paramList.add(new Param(MapTools.simpleMap("uuid", uuid), Sign.EQ));
                } else {
                    //manageone 使用名称查询
                    String hostname = String.valueOf(result.getMetric().get("id"));
                    paramList.add(new Param(MapTools.simpleMap("hostname", hostname), Sign.EQ));
                }
                GeneralResult serverResult = cmpServerService.list(new Pager(1, 1, paramList, null));
                return new TopBaseEntity(serverResult, hostName, result);
            });
            futures.add(future);
        }
        futures.forEach(future -> {
            try {
                TopBaseEntity topBaseEntity = future.get();
                TopParams topParams = new TopParams();
                String resultName = topBaseEntity.getResourceName();
                if (topBaseEntity.getGeneralResult().isSuccess()) {
                    List<Server> hosts = ((JSONObject) topBaseEntity.getGeneralResult().getData()).getJSONArray(Common.ROWS).toJavaList(Server.class);
                    if (!hosts.isEmpty()) {
                        Server server = hosts.get(0);
                        resultName = server.getName();
                        topParams.setId(server.getId());
                        topParams.setType(server.getVendorType());
                        topParams.setVendorId(server.getVendorId());
                        topParams.setIdentifier(server.getVendorType().equals(CloudProvider.MANAGEONE.name()) ? server.getName() : server.getManageIp());
                    }
                    params.put(resultName, topParams);
                }
                keys.add(resultName);
                data.add(topBaseEntity.getPrometheusResult().getValue()[1]);
            } catch (Exception e) {
                log.error("查询宿主机Top列表失败！" + e);
            }
        });
        return new LineChartResult(keys, new Values[]{new Values(name, data)}, params);
    }

    /**
     * 虚拟机Top图表
     *
     * @param name
     * @param expr
     * @return
     */
    private LineChartResult vmLineChart(HttpServletRequest request, String name, String expr, int limit) {
        // 获取当前时间戳
        String time = Long.toString(System.currentTimeMillis()).substring(0, 10);
        if ("".equals(expr)) {
            return null;
        }
        List<PrometheusResult> list = prometheusService.query(expr, time);
        List<String> keys = new ArrayList<>(list.size());
        List<Object> data = new ArrayList<>(list.size());
        List<Future<TopBaseEntity>> futures = new ArrayList<>();
        // id列表，用于作监控详情信息展示
        Map<String, TopParams> params = new HashMap<>(16, 0.75F);
        for (PrometheusResult result : list) {
            Future<TopBaseEntity> future = executorService.submit(() -> {
                CloudProvider vendorType = null;
                if (result.getMetric().containsKey("vendor_id")) {
                    GeneralResult<CloudVendor> vendorGeneralResult = cmpCloudVendorService.detail(Long.valueOf(String.valueOf(result.getMetric().get("vendor_id"))));
                    vendorType = CloudProvider.valueOf(vendorGeneralResult.getData().getType());
                }
                if (result.getMetric().containsKey("vendorId")) {
                    GeneralResult<CloudVendor> vendorGeneralResult = cmpCloudVendorService.detail(Long.valueOf(String.valueOf(result.getMetric().get("vendorId"))));
                    vendorType = CloudProvider.valueOf(vendorGeneralResult.getData().getType());
                }
                List<Param> paramList = new ArrayList<>();
                String vmName = null;
                if (result.getMetric().containsKey("vcenter")) {
                    //vmware 使用name查询
                    vmName = String.valueOf(result.getMetric().get("vmname"));
                    String uuid = String.valueOf(result.getMetric().get("uuid"));
                    paramList.add(new Param(MapTools.simpleMap("biosUuid", uuid), Sign.EQ));
                } else if (CloudProvider.SMARTX == vendorType){
                    // 使用localId查询
                    vmName = "smartx";
                    String instanceId = String.valueOf(result.getMetric().get("instance_id"));
                    paramList.add(new Param(MapTools.simpleMap("localId", instanceId), Sign.EQ));
                } else if (CloudProvider.CTSTACK == vendorType){
                    // 使用localId查询
                    vmName = "ctStack";
                    String instanceId = String.valueOf(result.getMetric().get("instanceId"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (result.getMetric().containsKey("vendor_id") && CloudProvider.OPENSTACK == vendorType) {
                    //openstack 使用instanceId查询
                    vmName = String.valueOf(result.getMetric().get("instanceName"));
                    String instanceId = String.valueOf(result.getMetric().get("instanceId"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (result.getMetric().containsKey("vm_uuid")) {
                    //KVM 使用vm_uuid查询
                    vmName = String.valueOf(result.getMetric().get("vmname"));
                    String instanceId = String.valueOf(result.getMetric().get("vm_uuid"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (result.getMetric().containsKey("vendorId") && CloudProvider.APSARASTACK == vendorType) {
                    //阿里专有云 使用vm查询
                    vmName = "apsarastack";
                    String instanceId = String.valueOf(result.getMetric().get("instanceId"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (result.getMetric().containsKey("vendorId") && CloudProvider.SMARTX == vendorType) {
                    //smartx 使用vm查询
                    vmName = String.valueOf(result.getMetric().get("vm"));
                    String instanceId = String.valueOf(result.getMetric().get("vm"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (result.getMetric().containsKey("EsCloud")) {
                    //easyStack 使用instanceId查询
                    vmName = String.valueOf(result.getMetric().get("id"));
                    String instanceId = String.valueOf(result.getMetric().get("id"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (result.getMetric().containsKey("urn")) {
                    vmName = String.valueOf(result.getMetric().get("urn"));
                    String instanceId = String.valueOf(result.getMetric().get("urn"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (result.getMetric().containsKey("instance_id")) {
                    //HCSO平台数据监控
                    String instanceId = String.valueOf(result.getMetric().get("instance_id"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                    vmName = "HCSO";
                } else if (CloudProvider.CNWARE == vendorType) {
                    //CNWARE平台数据监控
                    String instanceId = String.valueOf(result.getMetric().get("uuid"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (CloudProvider.ZSTACK == vendorType) {
                    //ZSTACK平台数据监控
                    String instanceId = String.valueOf(result.getMetric().get("VMUuid"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                }else if (CloudProvider.CLOUDOS == vendorType) {
                    //CLOUDOS平台数据监控
                    vmName = String.valueOf(result.getMetric().get("vmName"));
                    String instanceId = String.valueOf(result.getMetric().get("instanceId"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else {
                    //manageone 使用instanceId查询
                    vmName = String.valueOf(result.getMetric().get("name"));
                    String instanceId = String.valueOf(result.getMetric().get("id"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                }
                GeneralResult generalResult = cmpCloudServerService.list(new Pager(1, 1, paramList, null),
                        request.getHeader(Common.TOKEN));
                if (generalResult.isSuccess() && ((JSONObject) generalResult.getData()).getJSONArray(Common.ROWS).toJavaList(CloudServer.class).size() > 0) {
                    JSONObject serverList = (JSONObject) generalResult.getData();
                    CloudServer cloudServer = serverList.getJSONArray(Common.ROWS).toJavaList(CloudServer.class).get(0);
                    vmName = cloudServer.getName();
                }
                return new TopBaseEntity(generalResult, vmName, result);
            });
            futures.add(future);
        }
        for (Future<TopBaseEntity> future : futures) {
            try {
                TopBaseEntity topBaseEntity = future.get();
                TopParams topParams = new TopParams();
                if (topBaseEntity.getGeneralResult().isSuccess()) {
                    List<CloudServer> vms = ((JSONObject) topBaseEntity.getGeneralResult().getData()).getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
                    if (!vms.isEmpty()) {
                        CloudServer cloudServer = vms.get(0);
                        topParams.setId(cloudServer.getId());
                        topParams.setType(cloudServer.getVendorType());
                        topParams.setVendorId(cloudServer.getVendorId());
                        topParams.setHostName(cloudServer.getHostName());
                        topParams.setIdentifier(cloudServer.getVendorType().equals(CloudProvider.VMWARE.name()) ? cloudServer.getName() : cloudServer.getInstanceId());
                        params.put(topBaseEntity.getResourceName(), topParams);
                        if (CloudProvider.SMARTX.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                            keys.add(cloudServer.getName());
                        } else {
                            keys.add(topBaseEntity.getResourceName());
                        }
                        data.add(topBaseEntity.getPrometheusResult().getValue()[1]);
                    }
                }
            } catch (Exception e) {
                log.error("查询宿主机Top列表失败！" + e);
            }
        }
        if (keys.size() > 5) {
            keys = keys.subList(0, 5);
            data = data.subList(0, 5);
        }
        return new LineChartResult(keys, new Values[]{new Values(name, data)}, params);
    }

}
