package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.CephOsdModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.xsky.XskyCephOsdService;
import com.bocloud.dms.entity.CephOsd;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Xsky
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
@Slf4j
public class XskyCephOsdServiceImpl implements com.bocloud.cms.intf.XskyCephOsdService, InitializingBean {


    private final XskyCephOsdService xskyCephOsdService;
    private final ResourceAlarmStatusRepository alarmStatusRepository;
    private final ResourceAlarmStatusService resourceAlarmStatusService;

    @Autowired
    public XskyCephOsdServiceImpl(XskyCephOsdService xskyCephOsdService,
                                  ResourceAlarmStatusRepository alarmStatusRepository,
                                  ResourceAlarmStatusService resourceAlarmStatusService) {
        this.xskyCephOsdService = xskyCephOsdService;
        this.alarmStatusRepository = alarmStatusRepository;
        this.resourceAlarmStatusService = resourceAlarmStatusService;
    }

    @Override
    public GeneralResult list(Pager pager, RequestContext context) {
        GeneralResult result = xskyCephOsdService.list(pager);
        JSONObject data = (JSONObject) result.getData();
        List<CephOsdModel> list = data.getJSONArray(Common.ROWS).toJavaList(CephOsdModel.class);
        if (list.isEmpty()) {
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_XSKY_CEPH_OSD.name(), targets, context);
        for (CephOsdModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<CephOsd> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = xskyCephOsdService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CephOsd.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(CephOsd cephOsd, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("id", String.valueOf(cephOsd.getOsdId()));
        params.put("vendorId", String.valueOf(cephOsd.getVendorId()));
        return params;
    }

    @Override
    public CephOsd parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String osdId = String.valueOf(alert.getLabels().get("id"));
        String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
        List<Param> params = new ArrayList<>(2);
        params.add(new Param(MapTools.simpleMap("osdId", osdId), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        GeneralResult result = xskyCephOsdService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CephOsd> list = data.getJSONArray(Common.ROWS).toJavaList(CephOsd.class);
        Assert.isTrue(list != null && !list.isEmpty(), "cephOsd not found,osdId:" + osdId + ",vendorId:" + vendorId);
        return list.get(0);
    }

    @Override
    public void destroy(CephOsd object, String catalog) throws Exception {
        log.info("destroy ceph osd : " + object.getId());
        // TODO
    }

    @Override
    public void wrapAlarm(Alarm alarm, CephOsd cephOsd) {
        alarm.setTarget(cephOsd.getId());
        alarm.setTargetName(cephOsd.getName());
        alarm.setVendorId(cephOsd.getVendorId());
        alarm.setOwnerId(cephOsd.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, CephOsd object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_XSKY_CEPH_OSD, this);
    }
}

