package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.XskyHostService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.model.ServerModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.CmpServerService;
import com.bocloud.ims.entity.Server;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Xsky主机
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
@Slf4j
public class XskyHostServiceImpl implements XskyHostService, InitializingBean {


    @Autowired
    private CmpServerService cmpServerService;
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public GeneralResult list(Pager pager, RequestContext context) {
        GeneralResult result = cmpServerService.list(pager);
        JSONObject data = (JSONObject) result.getData();
        List<ServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(ServerModel.class);
        if (ListTool.isEmpty(list)) {
            return result;
        }

        Long vendorId = list.get(0).getVendorId();
        List<String> ids = list.stream().map(Server::getUuid).collect(Collectors.toList());
        List<PrometheusResult> cpuUtilList = prometheusService.query("round(xms_host_cpu_util{vendorId='" + vendorId + "',id=~'" + StringUtils.join(ids.toArray(), "|") + "'}*100,0.01)");
        List<PrometheusResult> memoryUsageList = prometheusService.query("round(xms_host_mem_usage_percent{vendorId='" + vendorId + "',id=~'" + StringUtils.join(ids.toArray(), "|") + "'}*100,0.01)");
        // CPU利用率
        list.forEach(server -> cpuUtilList.stream().filter(cpuUsage -> cpuUsage.getMetric().get("id").equals(server.getUuid()))
                .forEach(cpuUsage -> server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]))));
        // 内存利用率
        list.forEach(server -> memoryUsageList.stream().filter(memoryUsage -> memoryUsage.getMetric().get("id").equals(server.getUuid()))
                .forEach(memoryUsage -> server.setMemUsage(String.valueOf(memoryUsage.getValue()[1]))));

        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_XSKY_HOST.name(), targets, context);
        for (ServerModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<Server> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = cmpServerService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(Server.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(Server server, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("id", server.getUuid());
        params.put("vendorId", String.valueOf(server.getVendorId()));
        return params;
    }

    @Override
    public Server parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String uuid = String.valueOf(alert.getLabels().get("id"));
        String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
        List<Param> params = new ArrayList<>(2);
        params.add(new Param(MapTools.simpleMap("uuid", uuid), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        GeneralResult result = cmpServerService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        Assert.isTrue(list != null && !list.isEmpty(), "xsky server not found,uuid:" + uuid + ",vendorId:" + vendorId);
        return list.get(0);
    }

    @Override
    public void destroy(Server object, String catalog) throws Exception {
        log.info("destroy server : " + object.getId());
        //TODO
    }

    @Override
    public void wrapAlarm(Alarm alarm, Server server) {
        alarm.setTarget(server.getId());
        alarm.setTargetName(server.getManageIp());
        alarm.setVendorId(server.getVendorId());
        alarm.setOwnerId(server.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Server object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_XSKY_HOST, this);
    }
}

