package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.ObjectRouterModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.xsky.XskyObjectRouterService;
import com.bocloud.dms.entity.ObjectRouter;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Xsky
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
@Slf4j
public class XskyObjectRouterServiceImpl implements com.bocloud.cms.intf.XskyObjectRouterService, InitializingBean {


    private final XskyObjectRouterService xskyObjectRouterService;
    private final ResourceAlarmStatusRepository alarmStatusRepository;
    private final ResourceAlarmStatusService resourceAlarmStatusService;

    public XskyObjectRouterServiceImpl(XskyObjectRouterService xskyObjectRouterService,
                                       ResourceAlarmStatusRepository alarmStatusRepository,
                                       ResourceAlarmStatusService resourceAlarmStatusService) {
        this.xskyObjectRouterService = xskyObjectRouterService;
        this.alarmStatusRepository = alarmStatusRepository;
        this.resourceAlarmStatusService = resourceAlarmStatusService;
    }

    @Override
    public GeneralResult list(Pager pager, RequestContext context) {

        GeneralResult result = xskyObjectRouterService.list(pager);
        JSONObject data = (JSONObject) result.getData();
        List<ObjectRouterModel> list = data.getJSONArray(Common.ROWS).toJavaList(ObjectRouterModel.class);
        if (list.isEmpty()) {
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_XSKY_OBJECT_ROUTER.name(), targets, context);
        for (ObjectRouterModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<ObjectRouter> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = xskyObjectRouterService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(ObjectRouter.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(ObjectRouter objectRouter, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("id", String.valueOf(objectRouter.getObjectRouterId()));
        params.put("vendorId", String.valueOf(objectRouter.getVendorId()));
        return params;
    }

    @Override
    public ObjectRouter parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String objectRouterId = String.valueOf(alert.getLabels().get("id"));
        String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
        List<Param> params = new ArrayList<>(2);
        params.add(new Param(MapTools.simpleMap("objectRouterId", objectRouterId), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        GeneralResult result = xskyObjectRouterService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<ObjectRouter> list = data.getJSONArray(Common.ROWS).toJavaList(ObjectRouter.class);
        Assert.isTrue(list != null && !list.isEmpty(), "bucket not objectRouterId,bucketUuid:" + objectRouterId + ",vendorId:" + vendorId);
        return list.get(0);
    }

    @Override
    public void destroy(ObjectRouter objectRouter, String catalog) throws Exception {
        log.info("destroy object router : " + objectRouter.getId());
        // TODO
    }

    @Override
    public void wrapAlarm(Alarm alarm, ObjectRouter objectRouter) {
        alarm.setTarget(objectRouter.getId());
        alarm.setTargetName(objectRouter.getName());
        alarm.setVendorId(objectRouter.getVendorId());
        alarm.setOwnerId(objectRouter.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, ObjectRouter object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_XSKY_OBJECT_ROUTER, this);
    }
}

