package com.bocloud.cms.service.adapter;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.enums.Operator;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.model.ServerModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * KVM
 *
 * @author zhangyf
 */
@Component
public class KVMHostAdapter implements HostAdapter {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository resourceAlarmStatusRepository;

    public KVMHostAdapter(PrometheusService prometheusService, ResourceAlarmStatusRepository resourceAlarmStatusRepository) {
        this.prometheusService = prometheusService;
        this.resourceAlarmStatusRepository = resourceAlarmStatusRepository;
    }

    @Override
    public void list(List<ServerModel> list, RequestContext context) {
        List<String> hostIds = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.KVM.name()))
                .map(s -> String.valueOf(s.getId())).collect(Collectors.toList());
        if (ListTool.isEmpty(hostIds)) {
            return;
        }

        String hostId = StringUtils.join(hostIds.toArray(), "|");
        List<LabelParam> params = Collections.singletonList(new LabelParam("host_id", hostId, Operator.RE_EQ));
        String cpuExpr = "node_cpu_usage{}";
        String memoryExpr = "node_memory_usage{}";

        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.resourceAlarmStatusRepository.list(MonitorResourceType.MONITOR_KVM_HOST.name(), target, context);

        for (ServerModel server : list) {
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("host_id").equals(String.valueOf(server.getId()))) {
                    server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memUsage : memoryList) {
                if (memUsage.getMetric().get("host_id").equals(String.valueOf(server.getId()))) {
                    server.setMemUsage(String.valueOf(memUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            boolean alarmEnable = Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .noneMatch(a -> a.getResourceId().equals(server.getId().toString()));
            server.setAlarmEnable(alarmEnable);
        }
    }


}
