package com.bocloud.cms.service.adapter;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.model.ServerModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.entity.Server;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * Vmware
 *
 * @author zhangyf
 */
@RequiredArgsConstructor
@Component
public class VmwareHostAdapter implements HostAdapter {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository resourceAlarmStatusRepository;

    @Override
    public void list(List<ServerModel> list, RequestContext context) {
        List<String> vmnames = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.VMWARE.name()))
                .map(Server::getName).collect(Collectors.toList());
        if (ListTool.isEmpty(vmnames)) {
            return;
        }
        //拼接参数
        String names = StringUtils.join(vmnames.toArray(), "|");
        names = names.replaceAll("\\(", "\\\\\\\\(");
        names = names.replaceAll("\\)", "\\\\\\\\)");
        List<PrometheusResult> memory = prometheusService.query("round(vsphere_host_mem_usage_average{source=~'" + names + "'},0.01)");
        List<PrometheusResult> cpu = prometheusService.query("round(vsphere_host_cpu_usage_average{source=~'" + names + "',cpu='instance-total'},0.01)");

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.resourceAlarmStatusRepository.list(MonitorResourceType.MONITOR_VCENTER_HOST.name(), target, context);

        for (ServerModel server : list) {
            for (PrometheusResult cpuUsage : cpu) {
                if (cpuUsage.getMetric().get("source").equals(server.getName())) {
                    server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memUsage : memory) {
                if (memUsage.getMetric().get("source").equals(server.getName())) {
                    server.setMemUsage(String.valueOf(memUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            boolean alarmEnable = Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .noneMatch(a -> a.getResourceId().equals(server.getId().toString()));
            server.setAlarmEnable(alarmEnable);
        }
    }
}
