package com.bocloud.cms.service.adapter;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.enums.Operator;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Author: Wsx
 * @Date: 2023/5/18 14:25
 */
@Component
public class ZstackAdapter implements VmAdapter {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository resourceAlarmStatusRepository;

    public ZstackAdapter(PrometheusService prometheusService, ResourceAlarmStatusRepository resourceAlarmStatusRepository) {
        this.prometheusService = prometheusService;
        this.resourceAlarmStatusRepository = resourceAlarmStatusRepository;
    }

    @Override
    public void list(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.ZSTACK.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<LabelParam> params = Collections.singletonList(new LabelParam("VMUuid", instanceId, Operator.RE_EQ));
        String cpuExpr = "round(zstack_vm_CPUAverageUsedUtilization{},0.01)";
        String memoryExpr = "round(zstack_vm_MemoryUsedInPercent{},0.01)";

        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.resourceAlarmStatusRepository.list(MonitorResourceType.MONITOR_ZSTACK_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("VMUuid").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("VMUuid").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            boolean alarmEnable = Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .noneMatch(a -> a.getResourceId().equals(cloudServer.getId().toString()));
            cloudServer.setAlarmEnable(alarmEnable);
        }
    }

}
