package com.bocloud.cms.service.external;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.MonitorAuth;
import com.bocloud.cms.intf.AuthService;
import com.bocloud.cms.intf.external.RemoteManageOneService;
import com.bocloud.cms.model.LineChartResult;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.Result;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.text.DecimalFormat;
import java.util.*;

/**
 * ManageOne实体类
 *
 * @author zhangyf
 * @since 2019/11/25 周一 11:12
 */
@Service
public class RemoteManageOneServiceImpl implements RemoteManageOneService {

    private final AuthService authService;

    public RemoteManageOneServiceImpl(AuthService authService) {
        this.authService = authService;
    }

    @Override
    public JSONObject getInstanceDetail(Long vendorId, String className, String id) {
        Map<String, Object> auth = this.auth(vendorId);
        String url = auth.get("endpoint") + "/rest/cmdb/v1/instances/" + className + "/" + id;
        Result result = new HttpClient(PostDataFormat.RAW).get((Map<String, Object>) auth.get("header"), null, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return JSONObject.parseObject(result.getMessage());
    }

    @Override
    public List<JSONObject> listInstance(Long vendorId, String className, Map<String, Object> paramMap, List<String> contentSelector) {
        Map<String, Object> auth = this.auth(vendorId);
        String url = auth.get("endpoint") + "/rest/cmdb/v1/instances/" + className;
        Map<String, Object> params = new HashMap<>(16, 0.75F);
        params.put("pageSize", 1000);
        params.put("pageNo", 1);
        if (!ListTool.isEmpty(contentSelector)) {
            params.put("contentSelector", JSONObject.toJSONString(contentSelector));
        }
        List<Map<String, Object>> list = new ArrayList<>();
        Optional.ofNullable(paramMap).orElse(new HashMap<>(16, 0.75F)).forEach((key, value) -> {
            Map<String, Object> param = new HashMap<>(16, 0.75F);
            param.put("name", "key");
            param.put("value", value);
            list.add(MapTools.simpleMap("simple", param));
        });
        if (!ListTool.isEmpty(list)) {
            params.put("condition", MapTools.simpleMap("constraint", list));
        }
        Result result = new HttpClient(PostDataFormat.RAW).get((Map<String, Object>) auth.get("header"), params, url);
        JSONObject content = JSONObject.parseObject(result.getMessage());
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return JSONObject.parseArray(content.getString("objList"), JSONObject.class);
    }

    @Override
    public JSONObject resourcePoolStatistics(Long vendorId, String id) {
        Map<String, Object> auth = this.auth(vendorId);
        String url = auth.get("endpoint") + "/rest/capacity/v1/capbase/resource-pools/" + id + "/statistics";
        Result result = new HttpClient(PostDataFormat.RAW).get((Map<String, Object>) auth.get("header"), null, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return translateStatistics(JSONObject.parseObject(result.getMessage()));
    }

    @Override
    public JSONObject azoneStatistics(Long vendorId, String id) {
        Map<String, Object> auth = this.auth(vendorId);
        String url = auth.get("endpoint") + "/rest/capacity/v1/capbase/azones/" + id + "/statistics";
        Result result = new HttpClient(PostDataFormat.RAW).get((Map<String, Object>) auth.get("header"), null, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return translateStatistics(JSONObject.parseObject(result.getMessage()));
    }

    @Override
    public JSONObject resourcePoolCurrentCapacities(Long vendorId, String id) {
        Map<String, Object> auth = this.auth(vendorId);
        String url = auth.get("endpoint") + "/rest/capacity/v1/capbase/resource-pools/" + id + "/resource-types/cpu,memory,storage-pool/current-capacities";
        Result result = new HttpClient(PostDataFormat.RAW).get((Map<String, Object>) auth.get("header"), null, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return translateCurrentCapacities(JSONObject.parseObject(result.getMessage()));
    }

    @Override
    public JSONObject azoneCurrentCapacities(Long vendorId, String id) {
        Map<String, Object> auth = this.auth(vendorId);
        String url = auth.get("endpoint") + "/rest/capacity/v1/capbase/azones/" + id + "/resource-types/cpu,memory,storage-pool/current-capacities";
        Result result = new HttpClient(PostDataFormat.RAW).get((Map<String, Object>) auth.get("header"), null, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return translateCurrentCapacities(JSONObject.parseObject(result.getMessage()));
    }

    @Override
    public LineChartResult resourcePoolStoragePoolCapacities(Long vendorId, String id) {
        Map<String, Object> auth = this.auth(vendorId);
        String url = auth.get("endpoint") + "/rest/capacity/v1/capbase/resource-pools/" + id + "/resource-types/storage-pool/dimension-types/volumeTypeName/capacities";
        Result result = new HttpClient(PostDataFormat.RAW).get((Map<String, Object>) auth.get("header"), null, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return this.translateStoragePoolCapacities(JSONObject.parseObject(result.getMessage()));
    }

    @Override
    public LineChartResult azoneStoragePoolCapacities(Long vendorId, String id) {
        Map<String, Object> auth = this.auth(vendorId);
        String url = auth.get("endpoint") + "/rest/capacity/v1/capbase/azones/" + id + "/resource-types/storage-pool/dimension-types/volumeTypeName/capacities";
        Result result = new HttpClient(PostDataFormat.RAW).get((Map<String, Object>) auth.get("header"), null, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return this.translateStoragePoolCapacities(JSONObject.parseObject(result.getMessage()));
    }

    /**
     * 运维侧权限认证
     *
     * @param vendorId 平台id
     */
    public Map<String, Object> auth(Long vendorId) {
        // 获取FusionCloud运维侧地址及认证信息
        MonitorAuth monitorAuth = authService.detail(vendorId);
        Assert.notNull(monitorAuth, "vendor [" + vendorId + "] not found auth config info");
        String password;
        try {
            password = new AESEncryptor().decrypt(monitorAuth.getPassword(), null);
        } catch (Exception e) {
            throw new RuntimeException("decrypt password error: " + e.getMessage(), e);
        }
        HttpClient client = new HttpClient(PostDataFormat.RAW);
        String url = monitorAuth.getAddress() + "/rest/plat/smapp/v1/oauth/token";
        Map<String, Object> params = new HashMap<>(4);
        params.put("grantType", "password");
        params.put("userName", monitorAuth.getUsername());
        params.put("value", password);

        Map<String, Object> header = new HashMap<>(3);
        header.put("accept", "application/json");
        header.put("content-type", "application/json");
        Result result = client.put(header, params, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject content = JSONObject.parseObject(result.getMessage());


        Map<String, Object> defaultHeader = new HashMap<>(16, 0.75F);
        defaultHeader.put("Accept", "application/json");
        defaultHeader.put("Content-Type", "application/json;charset=UTF-8");
        defaultHeader.put("X-Auth-Token", content.getString("accessSession"));
        defaultHeader.put("roaRand", content.getString("roaRand"));

        Map<String, Object> auth = new HashMap<>(16, 0.75F);
        auth.put("endpoint", monitorAuth.getAddress());
        auth.put("header", defaultHeader);
        return auth;
    }

    /**
     * 宿主机、云主机数目统计格式转换
     *
     * @param jsonObject
     * @return
     */
    private JSONObject translateStatistics(JSONObject jsonObject) {
        List<Map<String, Object>> hostStatusList = new ArrayList<>();
        Map<String, Object> normal = new HashMap<>(16, 0.75F);
        normal.put("name", "正常");
        normal.put("value", jsonObject.getJSONObject("host").getInteger("normal"));
        Map<String, Object> poweroff = new HashMap<>(16, 0.75F);
        poweroff.put("name", "下电");
        poweroff.put("value", jsonObject.getJSONObject("host").getInteger("poweroff"));
        Map<String, Object> fault = new HashMap<>(16, 0.75F);
        fault.put("name", "故障");
        fault.put("value", jsonObject.getJSONObject("host").getInteger("fault"));
        Map<String, Object> other = new HashMap<>(16, 0.75F);
        other.put("name", "其他");
        other.put("value", jsonObject.getJSONObject("host").getInteger("other"));
        hostStatusList.add(normal);
        hostStatusList.add(poweroff);
        hostStatusList.add(fault);
        hostStatusList.add(other);

        List<Map<String, Object>> vmStatusList = new ArrayList<>();
        Map<String, Object> running = new HashMap<>(16, 0.75F);
        running.put("name", "正常");
        running.put("value", jsonObject.getJSONObject("vm").getInteger("running"));
        Map<String, Object> stopped = new HashMap<>(16, 0.75F);
        stopped.put("name", "停止");
        stopped.put("value", jsonObject.getJSONObject("vm").getInteger("stopped"));
        Map<String, Object> hibernated = new HashMap<>(16, 0.75F);
        hibernated.put("name", "休眠");
        hibernated.put("value", jsonObject.getJSONObject("vm").getInteger("hibernated"));
        Map<String, Object> vmOther = new HashMap<>(16, 0.75F);
        vmOther.put("name", "其他");
        vmOther.put("value", jsonObject.getJSONObject("vm").getInteger("other"));
        vmStatusList.add(running);
        vmStatusList.add(stopped);
        vmStatusList.add(hibernated);
        vmStatusList.add(vmOther);

        JSONObject data = new JSONObject();
        data.put("host", hostStatusList);
        data.put("vm", vmStatusList);
        return data;
    }

    /**
     * 容量数据格式转换
     *
     * @param jsonObject
     * @return
     */
    private JSONObject translateCurrentCapacities(JSONObject jsonObject) {
        List<Map<String, Object>> cpuOversubscriptionCapacity = new ArrayList<>();
        Map<String, Object> cpuAllocated = new HashMap<>(16, 0.75F);
        cpuAllocated.put("name", "分配量");
        cpuAllocated.put("value", jsonObject.getJSONObject("cpu").getJSONObject("oversubscriptionCapacity").getJSONObject("allocatedCapacity").getDoubleValue("capacityValue"));
        Map<String, Object> cpuFree = new HashMap<>(16, 0.75F);
        cpuFree.put("name", "剩余量");
        cpuFree.put("value", jsonObject.getJSONObject("cpu").getJSONObject("oversubscriptionCapacity").getJSONObject("freeCapacity").getDoubleValue("capacityValue"));
        cpuOversubscriptionCapacity.add(cpuAllocated);
        cpuOversubscriptionCapacity.add(cpuFree);

        List<Map<String, Object>> memActualCapacity = new ArrayList<>();
        Map<String, Object> memActualCapacityUsed = new HashMap<>(16, 0.75F);
        memActualCapacityUsed.put("name", "使用量");
        memActualCapacityUsed.put("value", new DecimalFormat(".00").format(jsonObject.getJSONObject("memory").getJSONObject("actualCapacity").getJSONObject("usedCapacity").getDoubleValue("capacityValue")));
        Map<String, Object> memActualCapacityFree = new HashMap<>(16, 0.75F);
        memActualCapacityFree.put("name", "剩余量");
        memActualCapacityFree.put("value", new DecimalFormat(".00").format(jsonObject.getJSONObject("memory").getJSONObject("actualCapacity").getJSONObject("freeCapacity").getDoubleValue("capacityValue")));
        memActualCapacity.add(memActualCapacityUsed);
        memActualCapacity.add(memActualCapacityFree);

        List<Map<String, Object>> memOversubscriptionCapacity = new ArrayList<>();
        Map<String, Object> memOversubscriptionCapacityUsed = new HashMap<>(16, 0.75F);
        memOversubscriptionCapacityUsed.put("name", "分配量");
        memOversubscriptionCapacityUsed.put("value", new DecimalFormat(".00").format(jsonObject.getJSONObject("memory").getJSONObject("oversubscriptionCapacity").getJSONObject("allocatedCapacity").getDoubleValue("capacityValue")));
        Map<String, Object> memOversubscriptionCapacityFree = new HashMap<>(16, 0.75F);
        memOversubscriptionCapacityFree.put("name", "剩余量");
        memOversubscriptionCapacityFree.put("value", new DecimalFormat(".00").format(jsonObject.getJSONObject("memory").getJSONObject("oversubscriptionCapacity").getJSONObject("freeCapacity").getDoubleValue("capacityValue")));
        memOversubscriptionCapacity.add(memOversubscriptionCapacityUsed);
        memOversubscriptionCapacity.add(memOversubscriptionCapacityFree);

        List<Map<String, Object>> storagePoolActualCapacity = new ArrayList<>();
        Map<String, Object> storagePoolActualCapacityUsed = new HashMap<>(16, 0.75F);
        Map<String, Object> storagePoolActualCapacityFree = new HashMap<>(16, 0.75F);
        storagePoolActualCapacityUsed.put("name", "使用量");
        storagePoolActualCapacityFree.put("name", "剩余量");
        storagePoolActualCapacityUsed.put("value", new DecimalFormat(".00").format(jsonObject.getJSONObject("storagePool").getJSONObject("actualCapacity").getJSONObject("usedCapacity").getDoubleValue("capacityValue")));
        storagePoolActualCapacityFree.put("value", new DecimalFormat(".00").format(jsonObject.getJSONObject("storagePool").getJSONObject("actualCapacity").getJSONObject("freeCapacity").getDoubleValue("capacityValue")));
        storagePoolActualCapacity.add(storagePoolActualCapacityUsed);
        storagePoolActualCapacity.add(storagePoolActualCapacityFree);

        List<Map<String, Object>> storagePoolOversubscriptionCapacity = new ArrayList<>();
        Map<String, Object> storagePoolOversubscriptionCapacityUsed = new HashMap<>(16, 0.75F);
        Map<String, Object> storagePoolOversubscriptionCapacityFree = new HashMap<>(16, 0.75F);
        storagePoolOversubscriptionCapacityUsed.put("name", "分配量");
        storagePoolOversubscriptionCapacityFree.put("name", "剩余量");
        storagePoolOversubscriptionCapacityUsed.put("value", new DecimalFormat(".00").format(jsonObject.getJSONObject("storagePool").getJSONObject("oversubscriptionCapacity").getJSONObject("allocatedCapacity").getDoubleValue("capacityValue")));
        storagePoolOversubscriptionCapacityFree.put("value", new DecimalFormat(".00").format(jsonObject.getJSONObject("storagePool").getJSONObject("oversubscriptionCapacity").getJSONObject("freeCapacity").getDoubleValue("capacityValue")));
        storagePoolOversubscriptionCapacity.add(storagePoolOversubscriptionCapacityUsed);
        storagePoolOversubscriptionCapacity.add(storagePoolOversubscriptionCapacityFree);

        JSONObject data = new JSONObject();
        JSONObject cpuData = new JSONObject();
        cpuData.put("total", jsonObject.getJSONObject("cpu").getJSONObject("actualCapacity").getJSONObject("totalCapacity").getString("capacityValue"));
        cpuData.put("oversubscriptionRatio", jsonObject.getJSONObject("cpu").getJSONObject("oversubscriptionCapacity").getInteger("oversubscriptionRatio"));
        cpuData.put("oversubscriptionCapacity", cpuOversubscriptionCapacity);

        JSONObject memData = new JSONObject();
        memData.put("oversubscriptionRatio", jsonObject.getJSONObject("memory").getJSONObject("oversubscriptionCapacity").getInteger("oversubscriptionRatio"));
        memData.put("actualCapacity", memActualCapacity);
        memData.put("oversubscriptionCapacity", memOversubscriptionCapacity);

        JSONObject storagePoolData = new JSONObject();
        storagePoolData.put("actualCapacity", storagePoolActualCapacity);
        storagePoolData.put("oversubscriptionCapacity", storagePoolOversubscriptionCapacity);

        data.put("cpu", cpuData);
        data.put("memory", memData);
        data.put("storagePool", storagePoolData);
        return data;
    }

    /**
     * 转换存储池数据格式
     *
     * @param jsonObject
     * @return
     */
    private LineChartResult translateStoragePoolCapacities(JSONObject jsonObject) {
        List<String> keys = new ArrayList<>();
        LineChartResult.Values actualUsedCapacity = new LineChartResult.Values("使用量", new ArrayList<>(), "actual");
        LineChartResult.Values actualFreeCapacity = new LineChartResult.Values("剩余量", new ArrayList<>(), "actual");
        LineChartResult.Values oversubscriptionAllocatedCapacity = new LineChartResult.Values("分配量", new ArrayList<>(), "oversubscription");
        LineChartResult.Values oversubscriptionFreeCapacity = new LineChartResult.Values("剩余量（超分）", new ArrayList<>(), "oversubscription");
        jsonObject.getJSONArray("capacityList").toJavaList(JSONObject.class).forEach(capacity -> {
            keys.add(capacity.getJSONArray("dimensions").getJSONObject(0).getString("dimensionValue"));
            actualUsedCapacity.getData().add(capacity.getJSONObject("capacity").getJSONObject("actualCapacity").getJSONObject("usedCapacity").getFloatValue("capacityValue"));
            actualFreeCapacity.getData().add(capacity.getJSONObject("capacity").getJSONObject("actualCapacity").getJSONObject("freeCapacity").getFloatValue("capacityValue"));
            oversubscriptionAllocatedCapacity.getData().add(capacity.getJSONObject("capacity").getJSONObject("oversubscriptionCapacity").getJSONObject("allocatedCapacity").getFloatValue("capacityValue"));
            oversubscriptionFreeCapacity.getData().add(capacity.getJSONObject("capacity").getJSONObject("oversubscriptionCapacity").getJSONObject("freeCapacity").getFloatValue("capacityValue"));
        });
        LineChartResult chart = new LineChartResult(keys, new LineChartResult.Values[]{actualUsedCapacity, actualFreeCapacity, oversubscriptionAllocatedCapacity, oversubscriptionFreeCapacity});
        chart.setUnit("GB");
        return chart;
    }

}
