package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.ims.entity.CloudServer;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * 云主机
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Component
public class CmpCloudServerService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    private static final Service SERVICE = Service.create(BoCloudService.CMP.getName());

    public CmpCloudServerService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    /**
     * 详情
     *
     * @param id 云主机ID
     * @return 云主机
     */
    public GeneralResult<CloudServer> detail(Long id) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms/" + id, HttpMethod.GET, headers, null);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseObject(data, CloudServer.class), "success");
    }

    /**
     * 列表查询
     *
     * @param pager 分页
     * @param token 如果为null，则使用internal_token内部调用查询
     * @return 云主机列表
     */
    public GeneralResult<?> list(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms", HttpMethod.GET, headers, pager.toMap(false));

        return remoteService.invoke();
    }
    /**
     * 列表查询
     *
     * @param pager 分页
     * @param token 如果为null，则使用internal_token内部调用查询
     * @return 云主机列表
     */
    public GeneralResult<?> listMysql(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/apsarastack/v1/rds/mysql", HttpMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }


    public GeneralResult list(Pager pager) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vendors", HttpMethod.GET, headers, pager.toMap());
        return remoteService.invoke();
    }


    /**
     * 磁盘列表查询
     *
     * @param pager 分页
     * @param token 如果为null，则使用internal_token内部调用查询
     * @return 磁盘列表
     */
    public GeneralResult<?> listOfVolume(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/volumes", HttpMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }

    public GeneralResult<?> listManagerCce(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/manageone/v1/containers/clusters", HttpMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }


    public GeneralResult<?> listManagerBms(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/manageone/v1/bms", HttpMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }

}
