package com.bocloud.cms.service.internal;

import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * 磁盘实体
 *
 * @author liuyuanyuan
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Component
public class DmsStorageService {

    private static final Service SERVICE = Service.create(BoCloudService.CMP.getName());

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    public DmsStorageService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }


    /**
     * 设备列表
     *
     * @param pager 分页
     * @param token Token
     * @return 设备列表
     */
    public GeneralResult<?> listDevices(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(6);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/storages", HttpMethod.GET, headers,
                pager.toMap(false));
        return remoteService.invoke();
    }


    /**
     * 磁盘列表
     *
     * @param pager 分页
     * @param token Token
     * @return 磁盘列表
     */
    public GeneralResult<?> listDisks(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(6);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/storages/disks", HttpMethod.GET, headers,
                pager.toMap(false));
        return remoteService.invoke();
    }

    /**
     * 存储卷列表
     *
     * @param pager 分页  分页
     * @param token  Token
     * @param action action
     * @return 存储卷列表
     */
    public GeneralResult<?> listVolumes(Pager pager, String token, String action) {
        Map<String, Object> headers = new HashMap<>(6);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        Map<String, Object> paramMap = pager.toMap(false);
        paramMap.put("action", action);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/volumes", HttpMethod.GET, headers, paramMap);
        return remoteService.invoke();
    }

}
