package com.bocloud.cms.service.internal.hmc;

import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * HMC
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Component
public class HmcService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    public HmcService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    /**
     * Server列表
     *
     * @param pager 分页
     * @return Server列表
     */
    public GeneralResult<?> listOfServer(Pager pager) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/plugins/hmc/v1/servers", HttpMethod.GET, headers,
                pager.toMap(false));
        return remoteService.invoke();
    }

    /**
     * Partition列表
     *
     * @param pager 分页
     * @return Partition列表
     */
    public GeneralResult<?> listOfPartition(Pager pager) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/plugins/hmc/v1/partitions", HttpMethod.GET, headers,
                pager.toMap(false));
        return remoteService.invoke();
    }

}
